"""Pytest configuration and shared fixtures for all tests."""

import os

import pytest

from meraki_client import MerakiClient


def pytest_configure(config: pytest.Config) -> None:
    """Register custom markers."""
    config.addinivalue_line(
        "markers",
        "mutating: marks tests that create/update/delete resources (deselect with '-m \"not mutating\"')",
    )


def pytest_collection_modifyitems(config: pytest.Config, items: list[pytest.Item]) -> None:
    """Skip mutating tests unless explicitly requested."""
    # If -m option was provided, let pytest handle it normally
    if config.getoption("-m"):
        return

    skip_mutating = pytest.mark.skip(
        reason="Mutating tests skipped by default. Use -m mutating to run."
    )
    for item in items:
        if "mutating" in item.keywords:
            item.add_marker(skip_mutating)


@pytest.fixture(scope="module")
def client() -> MerakiClient:
    """Create a Meraki client for testing."""
    api_key = os.environ.get("MERAKI_DASHBOARD_API_KEY_TESTS")
    if not api_key:
        pytest.skip("MERAKI_DASHBOARD_API_KEY_TESTS environment variable not set")
    return MerakiClient(api_key=api_key)


@pytest.fixture(scope="module")
def organization_id(client: MerakiClient) -> str:
    """Get the first organization ID for testing."""
    org_id = os.environ.get("ORGANIZATION_ID_TESTS")
    if org_id:
        return org_id
    orgs = client.organizations.get_organizations().collect()
    if not orgs:
        pytest.skip("No organizations available for testing")
    org = orgs[0]
    if not org.id:
        pytest.skip("No organization ID available for testing")
    return org.id


@pytest.fixture(scope="module")
def network_id(client: MerakiClient, organization_id: str) -> str:
    """Get the first network ID for testing."""
    env_network_id = os.environ.get("NETWORK_ID_TESTS")
    if env_network_id:
        return env_network_id
    networks = client.organizations.get_organization_networks(organization_id).collect()
    if not networks:
        pytest.skip("No networks available for testing")
    network = networks[0]
    if not network.id:
        pytest.skip("No network ID available for testing")
    return network.id


@pytest.fixture(scope="module")
def device_serial(client: MerakiClient, organization_id: str) -> str:
    """Get the first device serial for testing."""
    env_device_id = os.environ.get("DEVICE_ID_TESTS")
    if env_device_id:
        return env_device_id
    devices = client.organizations.get_organization_devices(organization_id).collect()
    if not devices:
        pytest.skip("No devices available for testing")
    device = devices[0]
    if not device.serial:
        pytest.skip("No device serial available for testing")
    return device.serial



"""Response schemas for cellular_gateway endpoints.

This module is automatically @generated from the Meraki API specification.
Do not edit this file manually.
"""

from __future__ import annotations

from datetime import datetime

from pydantic import Field, RootModel

from meraki_client.schemas._base import _BaseSchema


class CellularGatewayApnsItem(_BaseSchema):
    """Schema for CellularGatewayApnsItem."""

    name: str | None = None


class CellularGatewayDestinationsItem(_BaseSchema):
    """Schema for CellularGatewayDestinationsItem."""

    ip: str | None = None
    description: str | None = None
    default: bool | None = None


class CellularGatewayDevice(_BaseSchema):
    """Meraki Device properties."""

    name: str | None = None
    model: str | None = None
    serial: str | None = None
    url: str | None = None
    status: str | None = None


class CellularGatewayLogo(_BaseSchema):
    """Service Provider logo data."""

    url: str | None = None


class CellularGatewayNetwork(_BaseSchema):
    """Meraki Network properties."""

    id: str | None = None


class CellularGatewayPlansItem(_BaseSchema):
    """Schema for CellularGatewayPlansItem."""

    name: str | None = None
    type_: str | None = Field(default=None, validation_alias="type", serialization_alias="type")


class CellularGatewayProfilesItem(_BaseSchema):
    """Schema for CellularGatewayProfilesItem."""

    custom_apns: list[str] = Field(
        default_factory=list, validation_alias="customApns", serialization_alias="customApns"
    )
    iccid: str | None = None
    status: str | None = None
    service_provider: CellularGatewayServiceProvider | None = Field(
        default=None, validation_alias="serviceProvider", serialization_alias="serviceProvider"
    )


class CellularGatewayRoaming(_BaseSchema):
    """Roaming Status."""

    status: str | None = None


class CellularGatewayServiceProvider(_BaseSchema):
    """Service Provider information."""

    name: str | None = None
    plans: list[CellularGatewayPlansItem] = Field(default_factory=list)


class CellularGatewayServiceProvider2(_BaseSchema):
    """Service provider data."""

    name: str | None = None
    logo: CellularGatewayLogo | None = None


class CellularGatewaySignalStat(_BaseSchema):
    """Tower Signal Status (Cellular Signal Stats)."""

    rsrp: str | None = None
    rsrq: str | None = None


class CellularGatewayTerms(_BaseSchema):
    """Service provider terms."""

    content: str | None = None
    name: str | None = None


class CellularGatewayUplinksItem(_BaseSchema):
    """Schema for CellularGatewayUplinksItem."""

    interface: str | None = None
    status: str | None = None
    ip: str | None = None
    provider: str | None = None
    public_ip: str | None = Field(
        default=None, validation_alias="publicIp", serialization_alias="publicIp"
    )
    model: str | None = None
    signal_stat: CellularGatewaySignalStat | None = Field(
        default=None, validation_alias="signalStat", serialization_alias="signalStat"
    )
    mcc: str | None = None
    mnc: str | None = None
    roaming: CellularGatewayRoaming | None = None
    connection_type: str | None = Field(
        default=None, validation_alias="connectionType", serialization_alias="connectionType"
    )
    apn: str | None = None
    gateway: str | None = None
    dns1: str | None = None
    dns2: str | None = None
    signal_type: str | None = Field(
        default=None, validation_alias="signalType", serialization_alias="signalType"
    )
    mtu: int | None = None
    iccid: str | None = None
    imsi: str | None = None
    msisdn: str | None = None


class CreateOrganizationCellularGatewayEsimsServiceProvidersAccountResponse(_BaseSchema):
    """Response for createOrganizationCellularGatewayEsimsServiceProvidersAccount operation."""

    account_id: str | None = Field(
        default=None, validation_alias="accountId", serialization_alias="accountId"
    )
    last_updated_at: str | None = Field(
        default=None, validation_alias="lastUpdatedAt", serialization_alias="lastUpdatedAt"
    )
    service_provider: CellularGatewayServiceProvider2 | None = Field(
        default=None, validation_alias="serviceProvider", serialization_alias="serviceProvider"
    )
    title: str | None = None
    username: str | None = None


class CreateOrganizationCellularGatewayEsimsServiceProvidersAccountServiceProvider(_BaseSchema):
    """Service Provider information."""

    name: str | None = None


class CreateOrganizationCellularGatewayEsimsSwapResponse(_BaseSchema):
    """Response for createOrganizationCellularGatewayEsimsSwap operation."""

    eid: str
    iccid: str
    status: str


class CreateOrganizationCellularGatewayEsimsSwapSwapsItem(_BaseSchema):
    """Item schema for swaps."""

    eid: str
    target: CreateOrganizationCellularGatewayEsimsSwapSwapsItemTarget | None = None


class CreateOrganizationCellularGatewayEsimsSwapSwapsItemTarget(_BaseSchema):
    """Target Profile attributes."""

    account_id: str = Field(validation_alias="accountId", serialization_alias="accountId")
    communication_plan: str = Field(
        validation_alias="communicationPlan", serialization_alias="communicationPlan"
    )
    rate_plan: str = Field(validation_alias="ratePlan", serialization_alias="ratePlan")


class GetDeviceCellularGatewayLanResponse(_BaseSchema):
    """Response for getDeviceCellularGatewayLan operation."""

    device_name: str | None = Field(
        default=None, validation_alias="deviceName", serialization_alias="deviceName"
    )
    device_lan_ip: str | None = Field(
        default=None, validation_alias="deviceLanIp", serialization_alias="deviceLanIp"
    )
    device_subnet: str | None = Field(
        default=None, validation_alias="deviceSubnet", serialization_alias="deviceSubnet"
    )
    fixed_ip_assignments: list[GetDeviceCellularGatewayLanResponseFixedIpAssignmentsItem] = Field(
        default_factory=list,
        validation_alias="fixedIpAssignments",
        serialization_alias="fixedIpAssignments",
    )
    reserved_ip_ranges: list[GetDeviceCellularGatewayLanResponseReservedIpRangesItem] = Field(
        default_factory=list,
        validation_alias="reservedIpRanges",
        serialization_alias="reservedIpRanges",
    )


class GetDeviceCellularGatewayLanResponseFixedIpAssignmentsItem(_BaseSchema):
    """Schema for GetDeviceCellularGatewayLanResponseFixedIpAssignmentsItem."""

    name: str | None = None
    ip: str | None = None
    mac: str | None = None


class GetDeviceCellularGatewayLanResponseReservedIpRangesItem(_BaseSchema):
    """Schema for GetDeviceCellularGatewayLanResponseReservedIpRangesItem."""

    start: str | None = None
    end: str | None = None
    comment: str | None = None


class GetDeviceCellularGatewayPortForwardingRulesResponse(_BaseSchema):
    """Response for getDeviceCellularGatewayPortForwardingRules operation."""

    rules: list[GetDeviceCellularGatewayPortForwardingRulesResponseRulesItem] = Field(
        default_factory=list
    )


class GetDeviceCellularGatewayPortForwardingRulesResponseRulesItem(_BaseSchema):
    """Schema for GetDeviceCellularGatewayPortForwardingRulesResponseRulesItem."""

    name: str | None = None
    lan_ip: str | None = Field(default=None, validation_alias="lanIp", serialization_alias="lanIp")
    public_port: str | None = Field(
        default=None, validation_alias="publicPort", serialization_alias="publicPort"
    )
    local_port: str | None = Field(
        default=None, validation_alias="localPort", serialization_alias="localPort"
    )
    allowed_ips: list[str] = Field(
        default_factory=list, validation_alias="allowedIps", serialization_alias="allowedIps"
    )
    protocol: str | None = None
    access: str | None = None


class GetNetworkCellularGatewayConnectivityMonitoringDestinationsResponse(_BaseSchema):
    """Response for getNetworkCellularGatewayConnectivityMonitoringDestinations operation."""

    destinations: list[CellularGatewayDestinationsItem] = Field(default_factory=list)


class GetNetworkCellularGatewayDhcpResponse(_BaseSchema):
    """Response for getNetworkCellularGatewayDhcp operation."""

    dhcp_lease_time: str | None = Field(
        default=None, validation_alias="dhcpLeaseTime", serialization_alias="dhcpLeaseTime"
    )
    dns_nameservers: str | None = Field(
        default=None, validation_alias="dnsNameservers", serialization_alias="dnsNameservers"
    )
    dns_custom_nameservers: list[str] = Field(
        default_factory=list,
        validation_alias="dnsCustomNameservers",
        serialization_alias="dnsCustomNameservers",
    )


class GetNetworkCellularGatewaySubnetPoolResponse(_BaseSchema):
    """Response for getNetworkCellularGatewaySubnetPool operation."""

    deployment_mode: str | None = Field(
        default=None, validation_alias="deploymentMode", serialization_alias="deploymentMode"
    )
    cidr: str | None = None
    mask: int | None = None
    subnets: list[GetNetworkCellularGatewaySubnetPoolResponseSubnetsItem] = Field(
        default_factory=list
    )


class GetNetworkCellularGatewaySubnetPoolResponseSubnetsItem(_BaseSchema):
    """Schema for GetNetworkCellularGatewaySubnetPoolResponseSubnetsItem."""

    serial: str | None = None
    name: str | None = None
    appliance_ip: str | None = Field(
        default=None, validation_alias="applianceIp", serialization_alias="applianceIp"
    )
    subnet: str | None = None


class GetNetworkCellularGatewayUplinkResponse(_BaseSchema):
    """Response for getNetworkCellularGatewayUplink operation."""

    bandwidth_limits: GetNetworkCellularGatewayUplinkResponseBandwidthLimits | None = Field(
        default=None, validation_alias="bandwidthLimits", serialization_alias="bandwidthLimits"
    )


class GetNetworkCellularGatewayUplinkResponseBandwidthLimits(_BaseSchema):
    """The bandwidth settings for the 'cellular' uplink."""

    limit_up: int | None = Field(
        default=None, validation_alias="limitUp", serialization_alias="limitUp"
    )
    limit_down: int | None = Field(
        default=None, validation_alias="limitDown", serialization_alias="limitDown"
    )


class GetOrganizationCellularGatewayEsimsInventoryResponseItemsItem(_BaseSchema):
    """Schema for GetOrganizationCellularGatewayEsimsInventoryResponseItemsItem."""

    device: CellularGatewayDevice | None = None
    active: bool | None = None
    eid: str | None = None
    last_updated_at: str | None = Field(
        default=None, validation_alias="lastUpdatedAt", serialization_alias="lastUpdatedAt"
    )
    network: CellularGatewayNetwork | None = None
    profiles: list[CellularGatewayProfilesItem] = Field(default_factory=list)


class GetOrganizationCellularGatewayEsimsServiceProvidersAccountsCommunicationPlansResponseItemsItem(
    _BaseSchema
):
    """Schema for GetOrganizationCellularGatewayEsimsServiceProvidersAccountsCommunicationPlansResp
    onseItemsItem.
    """

    account_id: str | None = Field(
        default=None, validation_alias="accountId", serialization_alias="accountId"
    )
    apns: list[CellularGatewayApnsItem] = Field(default_factory=list)
    name: str | None = None


class GetOrganizationCellularGatewayEsimsServiceProvidersAccountsRatePlansResponseItemsItem(
    _BaseSchema
):
    """Schema for
    GetOrganizationCellularGatewayEsimsServiceProvidersAccountsRatePlansResponseItemsItem.
    """

    account_id: str | None = Field(
        default=None, validation_alias="accountId", serialization_alias="accountId"
    )
    name: str | None = None


class GetOrganizationCellularGatewayEsimsServiceProvidersAccountsResponse(
    RootModel[list["GetOrganizationCellularGatewayEsimsServiceProvidersAccountsResponseItem"]]
):
    """IList of Cellular Service Provider Accounts."""


class GetOrganizationCellularGatewayEsimsServiceProvidersAccountsResponseItem(_BaseSchema):
    """Schema for GetOrganizationCellularGatewayEsimsServiceProvidersAccountsResponseItem."""

    account_id: str | None = Field(
        default=None, validation_alias="accountId", serialization_alias="accountId"
    )
    last_updated_at: str | None = Field(
        default=None, validation_alias="lastUpdatedAt", serialization_alias="lastUpdatedAt"
    )
    service_provider: CellularGatewayServiceProvider2 | None = Field(
        default=None, validation_alias="serviceProvider", serialization_alias="serviceProvider"
    )
    title: str | None = None
    username: str | None = None


class GetOrganizationCellularGatewayEsimsServiceProvidersResponseItemsItem(_BaseSchema):
    """Schema for GetOrganizationCellularGatewayEsimsServiceProvidersResponseItemsItem."""

    name: str | None = None
    logo: CellularGatewayLogo | None = None
    is_bootstrap: bool | None = Field(
        default=None, validation_alias="isBootstrap", serialization_alias="isBootstrap"
    )
    terms: CellularGatewayTerms | None = None


class GetOrganizationCellularGatewayUplinkStatusesResponse(
    RootModel[list["GetOrganizationCellularGatewayUplinkStatusesResponseItem"]]
):
    """Response for getOrganizationCellularGatewayUplinkStatuses operation."""


class GetOrganizationCellularGatewayUplinkStatusesResponseItem(_BaseSchema):
    """Schema for GetOrganizationCellularGatewayUplinkStatusesResponseItem."""

    network_id: str | None = Field(
        default=None, validation_alias="networkId", serialization_alias="networkId"
    )
    serial: str | None = None
    model: str | None = None
    last_reported_at: datetime | None = Field(
        default=None, validation_alias="lastReportedAt", serialization_alias="lastReportedAt"
    )
    uplinks: list[CellularGatewayUplinksItem] = Field(default_factory=list)


class UpdateDeviceCellularGatewayLanFixedIpAssignmentsItem(_BaseSchema):
    """Item schema for fixedIpAssignments."""

    name: str | None = None
    ip: str
    mac: str


class UpdateDeviceCellularGatewayLanReservedIpRangesItem(_BaseSchema):
    """Item schema for reservedIpRanges."""

    start: str
    end: str
    comment: str


class UpdateDeviceCellularGatewayLanResponse(_BaseSchema):
    """Response for updateDeviceCellularGatewayLan operation."""

    device_name: str | None = Field(
        default=None, validation_alias="deviceName", serialization_alias="deviceName"
    )
    device_lan_ip: str | None = Field(
        default=None, validation_alias="deviceLanIp", serialization_alias="deviceLanIp"
    )
    device_subnet: str | None = Field(
        default=None, validation_alias="deviceSubnet", serialization_alias="deviceSubnet"
    )
    fixed_ip_assignments: list[GetDeviceCellularGatewayLanResponseFixedIpAssignmentsItem] = Field(
        default_factory=list,
        validation_alias="fixedIpAssignments",
        serialization_alias="fixedIpAssignments",
    )
    reserved_ip_ranges: list[GetDeviceCellularGatewayLanResponseReservedIpRangesItem] = Field(
        default_factory=list,
        validation_alias="reservedIpRanges",
        serialization_alias="reservedIpRanges",
    )


class UpdateDeviceCellularGatewayPortForwardingRulesResponse(_BaseSchema):
    """Response for updateDeviceCellularGatewayPortForwardingRules operation."""

    rules: list[GetDeviceCellularGatewayPortForwardingRulesResponseRulesItem] = Field(
        default_factory=list
    )


class UpdateDeviceCellularGatewayPortForwardingRulesRulesItem(_BaseSchema):
    """Item schema for rules."""

    name: str | None = None
    lan_ip: str = Field(validation_alias="lanIp", serialization_alias="lanIp")
    public_port: str = Field(validation_alias="publicPort", serialization_alias="publicPort")
    local_port: str = Field(validation_alias="localPort", serialization_alias="localPort")
    allowed_ips: list[str] = Field(
        default_factory=list, validation_alias="allowedIps", serialization_alias="allowedIps"
    )
    protocol: str
    access: str


class UpdateNetworkCellularGatewayConnectivityMonitoringDestinationsDestinationsItem(_BaseSchema):
    """Item schema for destinations."""

    ip: str
    description: str | None = None
    default: bool | None = None


class UpdateNetworkCellularGatewayConnectivityMonitoringDestinationsResponse(_BaseSchema):
    """Response for updateNetworkCellularGatewayConnectivityMonitoringDestinations operation."""

    destinations: list[CellularGatewayDestinationsItem] = Field(default_factory=list)


class UpdateNetworkCellularGatewayDhcpResponse(_BaseSchema):
    """Response for updateNetworkCellularGatewayDhcp operation."""

    dhcp_lease_time: str | None = Field(
        default=None, validation_alias="dhcpLeaseTime", serialization_alias="dhcpLeaseTime"
    )
    dns_nameservers: str | None = Field(
        default=None, validation_alias="dnsNameservers", serialization_alias="dnsNameservers"
    )
    dns_custom_nameservers: list[str] = Field(
        default_factory=list,
        validation_alias="dnsCustomNameservers",
        serialization_alias="dnsCustomNameservers",
    )


class UpdateNetworkCellularGatewaySubnetPoolResponse(_BaseSchema):
    """Response for updateNetworkCellularGatewaySubnetPool operation."""

    deployment_mode: str | None = Field(
        default=None, validation_alias="deploymentMode", serialization_alias="deploymentMode"
    )
    cidr: str | None = None
    mask: int | None = None
    subnets: list[GetNetworkCellularGatewaySubnetPoolResponseSubnetsItem] = Field(
        default_factory=list
    )


class UpdateNetworkCellularGatewayUplinkBandwidthLimits(_BaseSchema):
    """The bandwidth settings for the 'cellular' uplink."""

    limit_up: int | None = Field(
        default=None, validation_alias="limitUp", serialization_alias="limitUp"
    )
    limit_down: int | None = Field(
        default=None, validation_alias="limitDown", serialization_alias="limitDown"
    )


class UpdateNetworkCellularGatewayUplinkResponse(_BaseSchema):
    """Response for updateNetworkCellularGatewayUplink operation."""

    bandwidth_limits: GetNetworkCellularGatewayUplinkResponseBandwidthLimits | None = Field(
        default=None, validation_alias="bandwidthLimits", serialization_alias="bandwidthLimits"
    )


class UpdateOrganizationCellularGatewayEsimsInventoryResponse(_BaseSchema):
    """Response for updateOrganizationCellularGatewayEsimsInventory operation."""

    device: CellularGatewayDevice | None = None
    active: bool | None = None
    eid: str | None = None
    last_updated_at: str | None = Field(
        default=None, validation_alias="lastUpdatedAt", serialization_alias="lastUpdatedAt"
    )
    network: CellularGatewayNetwork | None = None
    profiles: list[CellularGatewayProfilesItem] = Field(default_factory=list)


class UpdateOrganizationCellularGatewayEsimsServiceProvidersAccountResponse(_BaseSchema):
    """Response for updateOrganizationCellularGatewayEsimsServiceProvidersAccount operation."""

    account_id: str | None = Field(
        default=None, validation_alias="accountId", serialization_alias="accountId"
    )
    last_updated_at: str | None = Field(
        default=None, validation_alias="lastUpdatedAt", serialization_alias="lastUpdatedAt"
    )
    service_provider: CellularGatewayServiceProvider2 | None = Field(
        default=None, validation_alias="serviceProvider", serialization_alias="serviceProvider"
    )
    title: str | None = None
    username: str | None = None


class UpdateOrganizationCellularGatewayEsimsSwapResponse(_BaseSchema):
    """Response for updateOrganizationCellularGatewayEsimsSwap operation."""

    eid: str
    iccid: str
    status: str



"""Tests for switch API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

from meraki_client import MerakiClient
from tests.generated.conftest import skip_on_unsupported


def test_get_device_switch_ports(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_switch_ports endpoint."""
    with skip_on_unsupported():
        client.switch.get_device_switch_ports(serial=device_serial)


def test_get_device_switch_ports_statuses(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_switch_ports_statuses endpoint."""
    with skip_on_unsupported():
        client.switch.get_device_switch_ports_statuses(serial=device_serial)


def test_get_device_switch_ports_statuses_packets(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_switch_ports_statuses_packets endpoint."""
    with skip_on_unsupported():
        client.switch.get_device_switch_ports_statuses_packets(serial=device_serial)


def test_get_device_switch_routing_interfaces(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_switch_routing_interfaces endpoint."""
    with skip_on_unsupported():
        client.switch.get_device_switch_routing_interfaces(serial=device_serial)


def test_get_device_switch_routing_static_routes(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_switch_routing_static_routes endpoint."""
    with skip_on_unsupported():
        client.switch.get_device_switch_routing_static_routes(serial=device_serial)


def test_get_device_switch_warm_spare(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_switch_warm_spare endpoint."""
    with skip_on_unsupported():
        client.switch.get_device_switch_warm_spare(serial=device_serial)


def test_get_network_switch_access_control_lists(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_access_control_lists endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_access_control_lists(network_id=network_id)


def test_get_network_switch_access_policies(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_access_policies endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_access_policies(network_id=network_id)


def test_get_network_switch_alternate_management_interface(
    client: MerakiClient, network_id: str
) -> None:
    """Test get_network_switch_alternate_management_interface endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_alternate_management_interface(network_id=network_id)


def test_get_network_switch_dhcp_v4_servers_seen(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_dhcp_v4_servers_seen endpoint."""
    with skip_on_unsupported():
        result = client.switch.get_network_switch_dhcp_v4_servers_seen(
            network_id=network_id
        ).collect()
    assert isinstance(result, list)


def test_get_network_switch_dhcp_server_policy(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_dhcp_server_policy endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_dhcp_server_policy(network_id=network_id)


def test_get_network_switch_dhcp_server_policy_arp_inspection_trusted_servers(
    client: MerakiClient, network_id: str
) -> None:
    """Test get_network_switch_dhcp_server_policy_arp_inspection_trusted_servers endpoint."""
    with skip_on_unsupported():
        result = client.switch.get_network_switch_dhcp_server_policy_arp_inspection_trusted_servers(
            network_id=network_id
        ).collect()
    assert isinstance(result, list)


def test_get_network_switch_dhcp_server_policy_arp_inspection_warnings_by_device(
    client: MerakiClient, network_id: str
) -> None:
    """Test get_network_switch_dhcp_server_policy_arp_inspection_warnings_by_device endpoint."""
    with skip_on_unsupported():
        result = (
            client.switch.get_network_switch_dhcp_server_policy_arp_inspection_warnings_by_device(
                network_id=network_id
            ).collect()
        )
    assert isinstance(result, list)


def test_get_network_switch_dscp_to_cos_mappings(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_dscp_to_cos_mappings endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_dscp_to_cos_mappings(network_id=network_id)


def test_get_network_switch_link_aggregations(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_link_aggregations endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_link_aggregations(network_id=network_id)


def test_get_network_switch_mtu(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_mtu endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_mtu(network_id=network_id)


def test_get_network_switch_port_schedules(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_port_schedules endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_port_schedules(network_id=network_id)


def test_get_network_switch_qos_rules(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_qos_rules endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_qos_rules(network_id=network_id)


def test_get_network_switch_qos_rules_order(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_qos_rules_order endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_qos_rules_order(network_id=network_id)


def test_get_network_switch_routing_multicast(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_routing_multicast endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_routing_multicast(network_id=network_id)


def test_get_network_switch_routing_multicast_rendezvous_points(
    client: MerakiClient, network_id: str
) -> None:
    """Test get_network_switch_routing_multicast_rendezvous_points endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_routing_multicast_rendezvous_points(network_id=network_id)


def test_get_network_switch_routing_ospf(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_routing_ospf endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_routing_ospf(network_id=network_id)


def test_get_network_switch_settings(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_settings endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_settings(network_id=network_id)


def test_get_network_switch_stacks(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_stacks endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_stacks(network_id=network_id)


def test_get_network_switch_storm_control(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_storm_control endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_storm_control(network_id=network_id)


def test_get_network_switch_stp(client: MerakiClient, network_id: str) -> None:
    """Test get_network_switch_stp endpoint."""
    with skip_on_unsupported():
        client.switch.get_network_switch_stp(network_id=network_id)


def test_get_organization_summary_switch_power_history(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_summary_switch_power_history endpoint."""
    with skip_on_unsupported():
        client.switch.get_organization_summary_switch_power_history(organization_id=organization_id)


def test_get_organization_switch_ports_by_switch(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_switch_ports_by_switch endpoint."""
    with skip_on_unsupported():
        result = client.switch.get_organization_switch_ports_by_switch(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_switch_ports_clients_overview_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_switch_ports_clients_overview_by_device endpoint."""
    with skip_on_unsupported():
        result = client.switch.get_organization_switch_ports_clients_overview_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_switch_ports_overview(client: MerakiClient, organization_id: str) -> None:
    """Test get_organization_switch_ports_overview endpoint."""
    with skip_on_unsupported():
        client.switch.get_organization_switch_ports_overview(organization_id=organization_id)


def test_get_organization_switch_ports_statuses_by_switch(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_switch_ports_statuses_by_switch endpoint."""
    with skip_on_unsupported():
        result = client.switch.get_organization_switch_ports_statuses_by_switch(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_switch_ports_topology_discovery_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_switch_ports_topology_discovery_by_device endpoint."""
    with skip_on_unsupported():
        result = client.switch.get_organization_switch_ports_topology_discovery_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_switch_ports_usage_history_by_device_by_interval(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_switch_ports_usage_history_by_device_by_interval endpoint."""
    with skip_on_unsupported():
        result = client.switch.get_organization_switch_ports_usage_history_by_device_by_interval(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)



"""Response schemas for devices endpoints.

This module is automatically @generated from the Meraki API specification.
Do not edit this file manually.
"""

from __future__ import annotations

from datetime import datetime
from typing import Any

from pydantic import Field, RootModel

from meraki_client.schemas._base import _BaseSchema


class BlinkDeviceLedsResponse(_BaseSchema):
    """Response for blinkDeviceLeds operation."""

    duration: int | None = None
    period: int | None = None
    duty: int | None = None


class CreateDeviceLiveToolsArpTableCallback(_BaseSchema):
    """Details for the callback. Please include either an httpServerId OR url and sharedSecret."""

    url: str | None = None
    shared_secret: str | None = Field(
        default=None, validation_alias="sharedSecret", serialization_alias="sharedSecret"
    )
    http_server: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="httpServer", serialization_alias="httpServer"
    )
    payload_template: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="payloadTemplate", serialization_alias="payloadTemplate"
    )


class CreateDeviceLiveToolsArpTableCallbackHttpServer(_BaseSchema):
    """The webhook receiver used for the callback webhook."""

    id: str | None = None


class CreateDeviceLiveToolsArpTableResponse(_BaseSchema):
    """Response for createDeviceLiveToolsArpTable operation."""

    arp_table_id: str | None = Field(
        default=None, validation_alias="arpTableId", serialization_alias="arpTableId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsArpTableResponseRequest | None = None
    status: str | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class CreateDeviceLiveToolsArpTableResponseCallback(_BaseSchema):
    """Information for callback used to send back results."""

    id: str | None = None
    url: str | None = None
    status: str | None = None


class CreateDeviceLiveToolsArpTableResponseRequest(_BaseSchema):
    """ARP table request parameters."""

    serial: str | None = None


class CreateDeviceLiveToolsCableTestCallback(_BaseSchema):
    """Details for the callback. Please include either an httpServerId OR url and sharedSecret."""

    url: str | None = None
    shared_secret: str | None = Field(
        default=None, validation_alias="sharedSecret", serialization_alias="sharedSecret"
    )
    http_server: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="httpServer", serialization_alias="httpServer"
    )
    payload_template: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="payloadTemplate", serialization_alias="payloadTemplate"
    )


class CreateDeviceLiveToolsCableTestResponse(_BaseSchema):
    """Response for createDeviceLiveToolsCableTest operation."""

    cable_test_id: str | None = Field(
        default=None, validation_alias="cableTestId", serialization_alias="cableTestId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsCableTestResponseRequest | None = None
    status: str | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class CreateDeviceLiveToolsCableTestResponseRequest(_BaseSchema):
    """Cable test request parameters."""

    serial: str | None = None
    ports: list[str] = Field(default_factory=list)


class CreateDeviceLiveToolsLedsBlinkCallback(_BaseSchema):
    """Details for the callback. Please include either an httpServerId OR url and sharedSecret."""

    url: str | None = None
    shared_secret: str | None = Field(
        default=None, validation_alias="sharedSecret", serialization_alias="sharedSecret"
    )
    http_server: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="httpServer", serialization_alias="httpServer"
    )
    payload_template: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="payloadTemplate", serialization_alias="payloadTemplate"
    )


class CreateDeviceLiveToolsLedsBlinkResponse(_BaseSchema):
    """Response for createDeviceLiveToolsLedsBlink operation."""

    leds_blink_id: str | None = Field(
        default=None, validation_alias="ledsBlinkId", serialization_alias="ledsBlinkId"
    )
    url: str | None = None
    status: str | None = None
    request: CreateDeviceLiveToolsLedsBlinkResponseRequest | None = None
    error: str | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class CreateDeviceLiveToolsLedsBlinkResponseRequest(_BaseSchema):
    """The parameters of the leds blink request."""

    serial: str | None = None
    duration: int | None = None


class CreateDeviceLiveToolsMacTableCallback(_BaseSchema):
    """Details for the callback. Please include either an httpServerId OR url and sharedSecret."""

    url: str | None = None
    shared_secret: str | None = Field(
        default=None, validation_alias="sharedSecret", serialization_alias="sharedSecret"
    )
    http_server: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="httpServer", serialization_alias="httpServer"
    )
    payload_template: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="payloadTemplate", serialization_alias="payloadTemplate"
    )


class CreateDeviceLiveToolsMacTableResponse(_BaseSchema):
    """Response for createDeviceLiveToolsMacTable operation."""

    mac_table_id: str | None = Field(
        default=None, validation_alias="macTableId", serialization_alias="macTableId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsArpTableResponseRequest | None = None
    status: str | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class CreateDeviceLiveToolsMulticastRoutingCallback(_BaseSchema):
    """Details for the callback. Please include either an httpServerId OR url and sharedSecret."""

    url: str | None = None
    shared_secret: str | None = Field(
        default=None, validation_alias="sharedSecret", serialization_alias="sharedSecret"
    )
    http_server: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="httpServer", serialization_alias="httpServer"
    )
    payload_template: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="payloadTemplate", serialization_alias="payloadTemplate"
    )


class CreateDeviceLiveToolsMulticastRoutingResponse(_BaseSchema):
    """Response for createDeviceLiveToolsMulticastRouting operation."""

    multicast_routing_id: str | None = Field(
        default=None,
        validation_alias="multicastRoutingId",
        serialization_alias="multicastRoutingId",
    )
    url: str | None = None
    request: CreateDeviceLiveToolsArpTableResponseRequest | None = None
    status: str | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class CreateDeviceLiveToolsPingCallback(_BaseSchema):
    """Details for the callback. Please include either an httpServerId OR url and sharedSecret."""

    url: str | None = None
    shared_secret: str | None = Field(
        default=None, validation_alias="sharedSecret", serialization_alias="sharedSecret"
    )
    http_server: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="httpServer", serialization_alias="httpServer"
    )
    payload_template: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="payloadTemplate", serialization_alias="payloadTemplate"
    )


class CreateDeviceLiveToolsPingDeviceCallback(_BaseSchema):
    """Details for the callback. Please include either an httpServerId OR url and sharedSecret."""

    url: str | None = None
    shared_secret: str | None = Field(
        default=None, validation_alias="sharedSecret", serialization_alias="sharedSecret"
    )
    http_server: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="httpServer", serialization_alias="httpServer"
    )
    payload_template: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="payloadTemplate", serialization_alias="payloadTemplate"
    )


class CreateDeviceLiveToolsPingDeviceResponse(_BaseSchema):
    """Response for createDeviceLiveToolsPingDevice operation."""

    ping_id: str | None = Field(
        default=None, validation_alias="pingId", serialization_alias="pingId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsPingDeviceResponseRequest | None = None
    status: str | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class CreateDeviceLiveToolsPingDeviceResponseRequest(_BaseSchema):
    """Ping request parameters."""

    serial: str | None = None
    count: int | None = None


class CreateDeviceLiveToolsPingResponse(_BaseSchema):
    """Response for createDeviceLiveToolsPing operation."""

    ping_id: str | None = Field(
        default=None, validation_alias="pingId", serialization_alias="pingId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsPingResponseRequest | None = None
    status: str | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class CreateDeviceLiveToolsPingResponseRequest(_BaseSchema):
    """Ping request parameters."""

    serial: str | None = None
    target: str | None = None
    count: int | None = None


class CreateDeviceLiveToolsThroughputTestCallback(_BaseSchema):
    """Details for the callback. Please include either an httpServerId OR url and sharedSecret."""

    url: str | None = None
    shared_secret: str | None = Field(
        default=None, validation_alias="sharedSecret", serialization_alias="sharedSecret"
    )
    http_server: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="httpServer", serialization_alias="httpServer"
    )
    payload_template: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="payloadTemplate", serialization_alias="payloadTemplate"
    )


class CreateDeviceLiveToolsThroughputTestResponse(_BaseSchema):
    """Response for createDeviceLiveToolsThroughputTest operation."""

    throughput_test_id: str | None = Field(
        default=None, validation_alias="throughputTestId", serialization_alias="throughputTestId"
    )
    url: str | None = None
    status: str | None = None
    result: CreateDeviceLiveToolsThroughputTestResponseResult | None = None
    request: CreateDeviceLiveToolsArpTableResponseRequest | None = None
    error: str | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class CreateDeviceLiveToolsThroughputTestResponseResult(_BaseSchema):
    """Result of the throughput test request."""

    speeds: DevicesResultSpeeds | None = None


class CreateDeviceLiveToolsWakeOnLanCallback(_BaseSchema):
    """Details for the callback. Please include either an httpServerId OR url and sharedSecret."""

    url: str | None = None
    shared_secret: str | None = Field(
        default=None, validation_alias="sharedSecret", serialization_alias="sharedSecret"
    )
    http_server: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="httpServer", serialization_alias="httpServer"
    )
    payload_template: CreateDeviceLiveToolsArpTableCallbackHttpServer | None = Field(
        default=None, validation_alias="payloadTemplate", serialization_alias="payloadTemplate"
    )


class CreateDeviceLiveToolsWakeOnLanResponse(_BaseSchema):
    """Response for createDeviceLiveToolsWakeOnLan operation."""

    wake_on_lan_id: str | None = Field(
        default=None, validation_alias="wakeOnLanId", serialization_alias="wakeOnLanId"
    )
    url: str | None = None
    status: str | None = None
    request: CreateDeviceLiveToolsWakeOnLanResponseRequest | None = None
    error: str | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class CreateDeviceLiveToolsWakeOnLanResponseRequest(_BaseSchema):
    """The parameters of the Wake-on-LAN request."""

    serial: str | None = None
    vlan_id: int | None = Field(
        default=None, validation_alias="vlanId", serialization_alias="vlanId"
    )
    mac: str | None = None


class DevicesAuthentication(_BaseSchema):
    """APN authentication configurations."""

    type_: str | None = Field(default=None, validation_alias="type", serialization_alias="type")
    username: str | None = None
    password: str | None = None


class DevicesResultSpeeds(_BaseSchema):
    """Shows the speeds (Mbps)."""

    downstream: float | None = None


class DevicesResultsLatencies(_BaseSchema):
    """Packet latency stats."""

    minimum: float | None = None
    average: float | None = None
    maximum: float | None = None


class DevicesResultsLoss(_BaseSchema):
    """Lost packets."""

    percentage: float | None = None


class DevicesResultsPairsItem(_BaseSchema):
    """Schema for DevicesResultsPairsItem."""

    index: int | None = None
    status: str | None = None
    length_meters: int | None = Field(
        default=None, validation_alias="lengthMeters", serialization_alias="lengthMeters"
    )


class DevicesResultsRepliesItem(_BaseSchema):
    """Schema for DevicesResultsRepliesItem."""

    sequence_id: int | None = Field(
        default=None, validation_alias="sequenceId", serialization_alias="sequenceId"
    )
    size: int | None = None
    latency: float | None = None


class DevicesSimsApnsItem(_BaseSchema):
    """Schema for DevicesSimsApnsItem."""

    name: str
    allowed_ip_types: list[str] = Field(
        validation_alias="allowedIpTypes", serialization_alias="allowedIpTypes"
    )
    authentication: DevicesAuthentication | None = None


class DevicesUsage(_BaseSchema):
    """Client usage data for sent and received."""

    sent: float | None = None
    recv: float | None = None


class DevicesWan1Vrf(_BaseSchema):
    """VRF associated with the management interface. If not provided, the default VRF is used."""

    name: str | None = None


class GetDeviceCellularSimsResponse(_BaseSchema):
    """Response for getDeviceCellularSims operation."""

    sims: list[GetDeviceCellularSimsResponseSimsItem] = Field(default_factory=list)
    sim_ordering: list[str] = Field(
        default_factory=list, validation_alias="simOrdering", serialization_alias="simOrdering"
    )
    sim_failover: GetDeviceCellularSimsResponseSimFailover | None = Field(
        default=None, validation_alias="simFailover", serialization_alias="simFailover"
    )


class GetDeviceCellularSimsResponseSimFailover(_BaseSchema):
    """SIM Failover settings."""

    enabled: bool | None = None
    timeout: int | None = None


class GetDeviceCellularSimsResponseSimsItem(_BaseSchema):
    """Schema for GetDeviceCellularSimsResponseSimsItem."""

    slot: str | None = None
    iccid: str | None = None
    imsi: str | None = None
    msisdn: str | None = None
    is_primary: bool | None = Field(
        default=None, validation_alias="isPrimary", serialization_alias="isPrimary"
    )
    status: str | None = None
    apns: list[DevicesSimsApnsItem] = Field(default_factory=list)


class GetDeviceClientsResponse(RootModel[list["GetDeviceClientsResponseItem"]]):
    """Response for getDeviceClients operation."""


class GetDeviceClientsResponseItem(_BaseSchema):
    """Schema for GetDeviceClientsResponseItem."""

    id: str | None = None
    mac: str | None = None
    description: str | None = None
    mdns_name: str | None = Field(
        default=None, validation_alias="mdnsName", serialization_alias="mdnsName"
    )
    dhcp_hostname: str | None = Field(
        default=None, validation_alias="dhcpHostname", serialization_alias="dhcpHostname"
    )
    user: str | None = None
    ip: str | None = None
    vlan: str | None = None
    named_vlan: str | None = Field(
        default=None, validation_alias="namedVlan", serialization_alias="namedVlan"
    )
    switchport: str | None = None
    adaptive_policy_group: str | None = Field(
        default=None,
        validation_alias="adaptivePolicyGroup",
        serialization_alias="adaptivePolicyGroup",
    )
    usage: DevicesUsage | None = None


class GetDeviceLiveToolsArpTableResponse(_BaseSchema):
    """Response for getDeviceLiveToolsArpTable operation."""

    arp_table_id: str | None = Field(
        default=None, validation_alias="arpTableId", serialization_alias="arpTableId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsArpTableResponseRequest | None = None
    status: str | None = None
    entries: list[GetDeviceLiveToolsArpTableResponseEntriesItem] = Field(default_factory=list)
    error: str | None = None


class GetDeviceLiveToolsArpTableResponseEntriesItem(_BaseSchema):
    """Schema for GetDeviceLiveToolsArpTableResponseEntriesItem."""

    ip: str | None = None
    mac: str | None = None
    vlan_id: int | None = Field(
        default=None, validation_alias="vlanId", serialization_alias="vlanId"
    )
    interface: str | None = None
    last_updated_at: datetime | None = Field(
        default=None, validation_alias="lastUpdatedAt", serialization_alias="lastUpdatedAt"
    )


class GetDeviceLiveToolsCableTestResponse(_BaseSchema):
    """Response for getDeviceLiveToolsCableTest operation."""

    cable_test_id: str | None = Field(
        default=None, validation_alias="cableTestId", serialization_alias="cableTestId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsCableTestResponseRequest | None = None
    status: str | None = None
    results: list[GetDeviceLiveToolsCableTestResponseResultsItem] = Field(default_factory=list)
    error: str | None = None


class GetDeviceLiveToolsCableTestResponseResultsItem(_BaseSchema):
    """Schema for GetDeviceLiveToolsCableTestResponseResultsItem."""

    port: str | None = None
    status: str | None = None
    speed_mbps: int | None = Field(
        default=None, validation_alias="speedMbps", serialization_alias="speedMbps"
    )
    error: str | None = None
    pairs: list[DevicesResultsPairsItem] = Field(default_factory=list)


class GetDeviceLiveToolsLedsBlinkResponse(_BaseSchema):
    """Response for getDeviceLiveToolsLedsBlink operation."""

    leds_blink_id: str | None = Field(
        default=None, validation_alias="ledsBlinkId", serialization_alias="ledsBlinkId"
    )
    url: str | None = None
    status: str | None = None
    request: CreateDeviceLiveToolsLedsBlinkResponseRequest | None = None
    error: str | None = None


class GetDeviceLiveToolsMacTableResponse(_BaseSchema):
    """Response for getDeviceLiveToolsMacTable operation."""

    mac_table_id: str | None = Field(
        default=None, validation_alias="macTableId", serialization_alias="macTableId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsArpTableResponseRequest | None = None
    status: str | None = None
    entries: list[GetDeviceLiveToolsMacTableResponseEntriesItem] = Field(default_factory=list)
    error: str | None = None


class GetDeviceLiveToolsMacTableResponseEntriesItem(_BaseSchema):
    """Schema for GetDeviceLiveToolsMacTableResponseEntriesItem."""

    mac: str | None = None
    port: str | None = None
    vlan_id: int | None = Field(
        default=None, validation_alias="vlanId", serialization_alias="vlanId"
    )


class GetDeviceLiveToolsMulticastRoutingResponse(_BaseSchema):
    """Response for getDeviceLiveToolsMulticastRouting operation."""

    multicast_routing_id: str | None = Field(
        default=None,
        validation_alias="multicastRoutingId",
        serialization_alias="multicastRoutingId",
    )
    url: str | None = None
    request: CreateDeviceLiveToolsArpTableResponseRequest | None = None
    status: str | None = None
    interfaces: list[GetDeviceLiveToolsMulticastRoutingResponseInterfacesItem] = Field(
        default_factory=list
    )
    routes: list[GetDeviceLiveToolsMulticastRoutingResponseRoutesItem] = Field(default_factory=list)
    error: str | None = None


class GetDeviceLiveToolsMulticastRoutingResponseInterfacesItem(_BaseSchema):
    """Schema for GetDeviceLiveToolsMulticastRoutingResponseInterfacesItem."""

    ip: str | None = None
    name: str | None = None
    subnet: str | None = None
    flags: list[str] = Field(default_factory=list)
    neighbors: list[str] = Field(default_factory=list)


class GetDeviceLiveToolsMulticastRoutingResponseRoutesItem(_BaseSchema):
    """Schema for GetDeviceLiveToolsMulticastRoutingResponseRoutesItem."""

    source: str | None = None
    group: str | None = None
    rendezvous_point: str | None = Field(
        default=None, validation_alias="rendezvousPoint", serialization_alias="rendezvousPoint"
    )
    incoming_interface_name: str | None = Field(
        default=None,
        validation_alias="incomingInterfaceName",
        serialization_alias="incomingInterfaceName",
    )
    outgoing_interface_names: list[str] = Field(
        default_factory=list,
        validation_alias="outgoingInterfaceNames",
        serialization_alias="outgoingInterfaceNames",
    )
    flags: list[str] = Field(default_factory=list)


class GetDeviceLiveToolsPingDeviceResponse(_BaseSchema):
    """Response for getDeviceLiveToolsPingDevice operation."""

    ping_id: str | None = Field(
        default=None, validation_alias="pingId", serialization_alias="pingId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsPingDeviceResponseRequest | None = None
    status: str | None = None
    results: GetDeviceLiveToolsPingResponseResults | None = None
    callback: CreateDeviceLiveToolsArpTableResponseCallback | None = None


class GetDeviceLiveToolsPingResponse(_BaseSchema):
    """Response for getDeviceLiveToolsPing operation."""

    ping_id: str | None = Field(
        default=None, validation_alias="pingId", serialization_alias="pingId"
    )
    url: str | None = None
    request: CreateDeviceLiveToolsPingResponseRequest | None = None
    status: str | None = None
    results: GetDeviceLiveToolsPingResponseResults | None = None


class GetDeviceLiveToolsPingResponseResults(_BaseSchema):
    """Results of the ping request."""

    sent: int | None = None
    received: int | None = None
    loss: DevicesResultsLoss | None = None
    latencies: DevicesResultsLatencies | None = None
    replies: list[DevicesResultsRepliesItem] = Field(default_factory=list)


class GetDeviceLiveToolsThroughputTestResponse(_BaseSchema):
    """Response for getDeviceLiveToolsThroughputTest operation."""

    throughput_test_id: str | None = Field(
        default=None, validation_alias="throughputTestId", serialization_alias="throughputTestId"
    )
    url: str | None = None
    status: str | None = None
    result: CreateDeviceLiveToolsThroughputTestResponseResult | None = None
    request: CreateDeviceLiveToolsArpTableResponseRequest | None = None
    error: str | None = None


class GetDeviceLiveToolsWakeOnLanResponse(_BaseSchema):
    """Response for getDeviceLiveToolsWakeOnLan operation."""

    wake_on_lan_id: str | None = Field(
        default=None, validation_alias="wakeOnLanId", serialization_alias="wakeOnLanId"
    )
    url: str | None = None
    status: str | None = None
    request: CreateDeviceLiveToolsWakeOnLanResponseRequest | None = None
    error: str | None = None


class GetDeviceLldpCdpResponse(_BaseSchema):
    """Response for getDeviceLldpCdp operation."""

    source_mac: str | None = Field(
        default=None, validation_alias="sourceMac", serialization_alias="sourceMac"
    )
    ports: dict[str, Any] | None = None


class GetDeviceLossAndLatencyHistoryResponse(
    RootModel[list["GetDeviceLossAndLatencyHistoryResponseItem"]]
):
    """Response for getDeviceLossAndLatencyHistory operation."""


class GetDeviceLossAndLatencyHistoryResponseItem(_BaseSchema):
    """Schema for GetDeviceLossAndLatencyHistoryResponseItem."""

    start_time: datetime | None = Field(
        default=None, validation_alias="startTime", serialization_alias="startTime"
    )
    end_time: datetime | None = Field(
        default=None, validation_alias="endTime", serialization_alias="endTime"
    )
    loss_percent: float | None = Field(
        default=None, validation_alias="lossPercent", serialization_alias="lossPercent"
    )
    latency_ms: float | None = Field(
        default=None, validation_alias="latencyMs", serialization_alias="latencyMs"
    )
    goodput: int | None = None
    jitter: float | None = None


class GetDeviceManagementInterfaceResponse(_BaseSchema):
    """Response for getDeviceManagementInterface operation."""

    ddns_hostnames: GetDeviceManagementInterfaceResponseDdnsHostnames | None = Field(
        default=None, validation_alias="ddnsHostnames", serialization_alias="ddnsHostnames"
    )
    wan1: GetDeviceManagementInterfaceResponseWan1 | None = None
    wan2: GetDeviceManagementInterfaceResponseWan1 | None = None


class GetDeviceManagementInterfaceResponseDdnsHostnames(_BaseSchema):
    """Dynamic DNS hostnames."""

    active_ddns_hostname: str | None = Field(
        default=None,
        validation_alias="activeDdnsHostname",
        serialization_alias="activeDdnsHostname",
    )
    ddns_hostname_wan1: str | None = Field(
        default=None, validation_alias="ddnsHostnameWan1", serialization_alias="ddnsHostnameWan1"
    )
    ddns_hostname_wan2: str | None = Field(
        default=None, validation_alias="ddnsHostnameWan2", serialization_alias="ddnsHostnameWan2"
    )


class GetDeviceManagementInterfaceResponseWan1(_BaseSchema):
    """WAN 1 settings."""

    wan_enabled: str | None = Field(
        default=None, validation_alias="wanEnabled", serialization_alias="wanEnabled"
    )
    using_static_ip: bool | None = Field(
        default=None, validation_alias="usingStaticIp", serialization_alias="usingStaticIp"
    )
    static_ip: str | None = Field(
        default=None, validation_alias="staticIp", serialization_alias="staticIp"
    )
    static_subnet_mask: str | None = Field(
        default=None, validation_alias="staticSubnetMask", serialization_alias="staticSubnetMask"
    )
    static_gateway_ip: str | None = Field(
        default=None, validation_alias="staticGatewayIp", serialization_alias="staticGatewayIp"
    )
    static_dns: list[str] = Field(
        default_factory=list, validation_alias="staticDns", serialization_alias="staticDns"
    )
    vlan: int | None = None
    vrf: DevicesWan1Vrf | None = None


class GetDeviceResponse(_BaseSchema):
    """Response for getDevice operation."""

    name: str | None = None
    lat: float | None = None
    lng: float | None = None
    address: str | None = None
    notes: str | None = None
    tags: list[str] = Field(default_factory=list)
    network_id: str | None = Field(
        default=None, validation_alias="networkId", serialization_alias="networkId"
    )
    serial: str | None = None
    model: str | None = None
    mac: str | None = None
    lan_ip: str | None = Field(default=None, validation_alias="lanIp", serialization_alias="lanIp")
    firmware: str | None = None
    floor_plan_id: str | None = Field(
        default=None, validation_alias="floorPlanId", serialization_alias="floorPlanId"
    )
    details: list[GetDeviceResponseDetailsItem] = Field(default_factory=list)
    beacon_id_params: GetDeviceResponseBeaconIdParams | None = Field(
        default=None, validation_alias="beaconIdParams", serialization_alias="beaconIdParams"
    )


class GetDeviceResponseBeaconIdParams(_BaseSchema):
    """Beacon Id parameters with an identifier and major and minor versions."""

    uuid: str | None = None
    major: int | None = None
    minor: int | None = None


class GetDeviceResponseDetailsItem(_BaseSchema):
    """Schema for GetDeviceResponseDetailsItem."""

    name: str | None = None
    value: str | None = None


class RebootDeviceResponse(_BaseSchema):
    """Response for rebootDevice operation."""

    success: bool | None = None


class UpdateDeviceCellularSimsResponse(_BaseSchema):
    """Response for updateDeviceCellularSims operation."""

    sims: list[GetDeviceCellularSimsResponseSimsItem] = Field(default_factory=list)
    sim_ordering: list[str] = Field(
        default_factory=list, validation_alias="simOrdering", serialization_alias="simOrdering"
    )
    sim_failover: GetDeviceCellularSimsResponseSimFailover | None = Field(
        default=None, validation_alias="simFailover", serialization_alias="simFailover"
    )


class UpdateDeviceCellularSimsSimFailover(_BaseSchema):
    """SIM Failover settings."""

    enabled: bool | None = None
    timeout: int | None = None


class UpdateDeviceCellularSimsSimsItem(_BaseSchema):
    """Item schema for sims."""

    slot: str | None = None
    is_primary: bool | None = Field(
        default=None, validation_alias="isPrimary", serialization_alias="isPrimary"
    )
    apns: list[DevicesSimsApnsItem] = Field(default_factory=list)
    sim_order: int | None = Field(
        default=None, validation_alias="simOrder", serialization_alias="simOrder"
    )


class UpdateDeviceManagementInterfaceResponse(_BaseSchema):
    """Response for updateDeviceManagementInterface operation."""

    ddns_hostnames: GetDeviceManagementInterfaceResponseDdnsHostnames | None = Field(
        default=None, validation_alias="ddnsHostnames", serialization_alias="ddnsHostnames"
    )
    wan1: GetDeviceManagementInterfaceResponseWan1 | None = None
    wan2: GetDeviceManagementInterfaceResponseWan1 | None = None


class UpdateDeviceManagementInterfaceWan1(_BaseSchema):
    """WAN 1 settings."""

    wan_enabled: str | None = Field(
        default=None, validation_alias="wanEnabled", serialization_alias="wanEnabled"
    )
    using_static_ip: bool | None = Field(
        default=None, validation_alias="usingStaticIp", serialization_alias="usingStaticIp"
    )
    static_ip: str | None = Field(
        default=None, validation_alias="staticIp", serialization_alias="staticIp"
    )
    static_gateway_ip: str | None = Field(
        default=None, validation_alias="staticGatewayIp", serialization_alias="staticGatewayIp"
    )
    static_subnet_mask: str | None = Field(
        default=None, validation_alias="staticSubnetMask", serialization_alias="staticSubnetMask"
    )
    static_dns: list[str] = Field(
        default_factory=list, validation_alias="staticDns", serialization_alias="staticDns"
    )
    vlan: int | None = None


class UpdateDeviceManagementInterfaceWan2(_BaseSchema):
    """WAN 2 settings (only for MX devices)."""

    wan_enabled: str | None = Field(
        default=None, validation_alias="wanEnabled", serialization_alias="wanEnabled"
    )
    using_static_ip: bool | None = Field(
        default=None, validation_alias="usingStaticIp", serialization_alias="usingStaticIp"
    )
    static_ip: str | None = Field(
        default=None, validation_alias="staticIp", serialization_alias="staticIp"
    )
    static_gateway_ip: str | None = Field(
        default=None, validation_alias="staticGatewayIp", serialization_alias="staticGatewayIp"
    )
    static_subnet_mask: str | None = Field(
        default=None, validation_alias="staticSubnetMask", serialization_alias="staticSubnetMask"
    )
    static_dns: list[str] = Field(
        default_factory=list, validation_alias="staticDns", serialization_alias="staticDns"
    )
    vlan: int | None = None


class UpdateDeviceResponse(_BaseSchema):
    """Response for updateDevice operation."""

    name: str | None = None
    lat: float | None = None
    lng: float | None = None
    address: str | None = None
    notes: str | None = None
    tags: list[str] = Field(default_factory=list)
    network_id: str | None = Field(
        default=None, validation_alias="networkId", serialization_alias="networkId"
    )
    serial: str | None = None
    model: str | None = None
    mac: str | None = None
    lan_ip: str | None = Field(default=None, validation_alias="lanIp", serialization_alias="lanIp")
    firmware: str | None = None
    floor_plan_id: str | None = Field(
        default=None, validation_alias="floorPlanId", serialization_alias="floorPlanId"
    )
    details: list[GetDeviceResponseDetailsItem] = Field(default_factory=list)
    beacon_id_params: GetDeviceResponseBeaconIdParams | None = Field(
        default=None, validation_alias="beaconIdParams", serialization_alias="beaconIdParams"
    )



"""Shared utility functions for code generation."""

import os
import re
import tomllib
from dataclasses import dataclass, field

from codegen.constants import RESERVED_NAMES, SPEC_OVERRIDES_FILE

_LOWER_UPPER_RE = re.compile(r"([a-z0-9])([A-Z])")
_ACRONYM_RE = re.compile(r"([A-Z]+)([A-Z][a-z])")


def to_snake_case(name: str) -> str:
    """Convert camelCase or PascalCase to snake_case, preserving acronyms."""
    name = _LOWER_UPPER_RE.sub(r"\1_\2", name)
    # Insert underscore between acronym and next word: VPNPeers -> VPN_Peers
    name = _ACRONYM_RE.sub(r"\1_\2", name)
    return name.lower()


def capitalize_first(name: str) -> str:
    """Capitalize the first character, leaving the rest unchanged."""
    return name[:1].upper() + name[1:] if name else ""


def escape_reserved_name(name: str) -> str:
    """Append underscore if name is a Python keyword or builtin."""
    return f"{name}_" if name in RESERVED_NAMES else name


def sanitize_text(text: str) -> str:
    """Clean up text from OpenAPI spec.

    - Replaces non-breaking spaces with regular spaces
    - Normalizes whitespace
    - Ensures text ends with a period
    """
    text = text.replace("\u00a0", " ").replace("\u2007", " ").replace("\u202f", " ")
    text = " ".join(text.split())
    if not text.endswith("."):
        text += "."
    return text


@dataclass
class SpecOverrides:
    """All spec overrides loaded from TOML configuration."""

    force_array_response: set[str] = field(default_factory=set)
    force_paginated: set[str] = field(default_factory=set)
    force_paginated_items_schema: set[str] = field(default_factory=set)
    optional_response: set[str] = field(default_factory=set)
    skip_tests: set[str] = field(default_factory=set)
    response_fields: dict[str, dict[str, str]] = field(default_factory=dict)
    required_fields: dict[str, set[str]] = field(default_factory=dict)
    extra_fields: dict[str, dict[str, str]] = field(default_factory=dict)


def load_spec_overrides() -> SpecOverrides:
    """Load spec overrides from TOML configuration file."""
    if not os.path.exists(SPEC_OVERRIDES_FILE):
        return SpecOverrides()

    with open(SPEC_OVERRIDES_FILE, "rb") as f:
        data = tomllib.load(f)

    response_fields: dict[str, dict[str, str]] = {}
    required_fields: dict[str, set[str]] = {}
    extra_fields: dict[str, dict[str, str]] = {}
    for key, value in data.items():
        if isinstance(value, dict):
            if "response" in value:
                response_fields[key] = value["response"]
            if "required" in value:
                required_fields[key] = set(value["required"])
            if "extra_fields" in value:
                extra_fields[key] = value["extra_fields"]

    return SpecOverrides(
        force_array_response=set(data.get("force_array_response", [])),
        force_paginated=set(data.get("force_paginated", [])),
        force_paginated_items_schema=set(data.get("force_paginated_items_schema", [])),
        optional_response=set(data.get("optional_response", [])),
        skip_tests=set(data.get("skip_tests", [])),
        response_fields=response_fields,
        required_fields=required_fields,
        extra_fields=extra_fields,
    )



"""Response schemas for spaces endpoints.

This module is automatically @generated from the Meraki API specification.
Do not edit this file manually.
"""

from __future__ import annotations

from pydantic import Field

from meraki_client.schemas._base import _BaseSchema


class GetOrganizationSpacesIntegrateStatusResponse(_BaseSchema):
    """Response for getOrganizationSpacesIntegrateStatus operation."""

    status: bool | None = None
    states: list[str] = Field(default_factory=list)
    email: str | None = None
    account_name: str | None = Field(
        default=None, validation_alias="accountName", serialization_alias="accountName"
    )
    account_type: str | None = Field(
        default=None, validation_alias="accountType", serialization_alias="accountType"
    )


class RemoveOrganizationSpacesIntegrationResponse(_BaseSchema):
    """Response for removeOrganizationSpacesIntegration operation."""

    status: bool | None = None
    message: str | None = None



"""ActionBatchCamera API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

import urllib.parse
from typing import Any

from meraki_client.schemas import (
    CreateOrganizationActionBatchActionsItem,
    UpdateDeviceCameraCustomAnalyticsParametersItem,
    UpdateDeviceCameraSenseAudioDetection,
    UpdateDeviceCameraWirelessProfilesIds,
)
from meraki_client.types import (
    UpdateDeviceCameraQualityAndRetentionMotionDetectorVersion,
    UpdateDeviceCameraQualityAndRetentionQuality,
    UpdateDeviceCameraQualityAndRetentionResolution,
)


class ActionBatchCamera:
    """ActionBatchCamera class."""

    def __init__(self) -> None:
        pass

    def update_device_camera_custom_analytics(
        self,
        serial: str,
        *,
        enabled: bool | None = None,
        artifact_id: str | None = None,
        parameters: list[UpdateDeviceCameraCustomAnalyticsParametersItem] | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update custom analytics settings for a camera.

        [API documentation: updateDeviceCameraCustomAnalytics](https://developer.cisco.com/meraki/api-v1/#!update-device-camera-custom-analytics)

        Args:
            serial: Serial.
            enabled: Enable custom analytics.
            artifact_id: The ID of the custom analytics artifact.
            parameters: Parameters for the custom analytics workload.

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/camera/customAnalytics"

        payload: dict[str, Any] = {}
        if enabled is not None:
            payload["enabled"] = enabled
        if artifact_id is not None:
            payload["artifactId"] = artifact_id
        if parameters is not None:
            payload["parameters"] = [
                item.model_dump(by_alias=True, exclude_none=True) for item in parameters
            ]

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_device_camera_quality_and_retention(
        self,
        serial: str,
        *,
        profile_id: str | None = None,
        motion_based_retention_enabled: bool | None = None,
        audio_recording_enabled: bool | None = None,
        restricted_bandwidth_mode_enabled: bool | None = None,
        quality: UpdateDeviceCameraQualityAndRetentionQuality | None = None,
        resolution: UpdateDeviceCameraQualityAndRetentionResolution | None = None,
        motion_detector_version: UpdateDeviceCameraQualityAndRetentionMotionDetectorVersion
        | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update quality and retention settings for the given camera.

        [API documentation: updateDeviceCameraQualityAndRetention](https://developer.cisco.com/meraki/api-v1/#!update-device-camera-quality-and-retention)

        Args:
            serial: Serial.
            profile_id: The ID of a quality and retention profile to assign to the camera. The
                profile's settings will override all of the per-camera quality and
                retention settings. If the value of this parameter is null, any existing
                profile will be unassigned from the camera.
            motion_based_retention_enabled: Boolean indicating if motion-based retention is
                enabled(true) or disabled(false) on the camera.
            audio_recording_enabled: Boolean indicating if audio recording is enabled(true) or
                disabled(false) on the camera.
            restricted_bandwidth_mode_enabled: Boolean indicating if restricted bandwidth is
                enabled(true) or disabled(false) on the camera. This setting does not
                apply to MV2 cameras.
            quality: Quality of the camera. Can be one of 'Standard', 'High', 'Enhanced' or 'Ultra'.
                Not all qualities are supported by every camera model.
            resolution: Resolution of the camera. Can be one of '1280x720', '1920x1080',
                '1080x1080', '2112x2112', '2880x2880', '2688x1512' or '3840x2160'.Not
                all resolutions are supported by every camera model.
            motion_detector_version: The version of the motion detector that will be used by the
                camera. Only applies to Gen 2 cameras. Defaults to v2.

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/camera/qualityAndRetention"

        payload: dict[str, Any] = {}
        if profile_id is not None:
            payload["profileId"] = profile_id
        if motion_based_retention_enabled is not None:
            payload["motionBasedRetentionEnabled"] = motion_based_retention_enabled
        if audio_recording_enabled is not None:
            payload["audioRecordingEnabled"] = audio_recording_enabled
        if restricted_bandwidth_mode_enabled is not None:
            payload["restrictedBandwidthModeEnabled"] = restricted_bandwidth_mode_enabled
        if quality is not None:
            payload["quality"] = quality
        if resolution is not None:
            payload["resolution"] = resolution
        if motion_detector_version is not None:
            payload["motionDetectorVersion"] = motion_detector_version

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_device_camera_sense(
        self,
        serial: str,
        *,
        sense_enabled: bool | None = None,
        mqtt_broker_id: str | None = None,
        audio_detection: UpdateDeviceCameraSenseAudioDetection | None = None,
        detection_model_id: str | None = None,
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update sense settings for the given camera.

        [API documentation: updateDeviceCameraSense](https://developer.cisco.com/meraki/api-v1/#!update-device-camera-sense)

        Args:
            serial: Serial.
            sense_enabled: Boolean indicating if sense(license) is enabled(true) or disabled(false)
                on the camera.
            mqtt_broker_id: The ID of the MQTT broker to be enabled on the camera. A value of null
                will disable MQTT on the camera.
            audio_detection: The details of the audio detection config.
            detection_model_id: The ID of the object detection model.

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/camera/sense"

        payload: dict[str, Any] = {}
        if sense_enabled is not None:
            payload["senseEnabled"] = sense_enabled
        if mqtt_broker_id is not None:
            payload["mqttBrokerId"] = mqtt_broker_id
        if audio_detection is not None:
            payload["audioDetection"] = audio_detection.model_dump(by_alias=True, exclude_none=True)
        if detection_model_id is not None:
            payload["detectionModelId"] = detection_model_id

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_device_camera_video_settings(
        self, serial: str, *, external_rtsp_enabled: bool | None = None
    ) -> CreateOrganizationActionBatchActionsItem:
        """Update video settings for the given camera.

        [API documentation: updateDeviceCameraVideoSettings](https://developer.cisco.com/meraki/api-v1/#!update-device-camera-video-settings)

        Args:
            serial: Serial.
            external_rtsp_enabled: Boolean indicating if external rtsp stream is exposed.

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/camera/video/settings"

        payload: dict[str, Any] = {}
        if external_rtsp_enabled is not None:
            payload["externalRtspEnabled"] = external_rtsp_enabled

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )

    def update_device_camera_wireless_profiles(
        self, *, serial: str, ids: UpdateDeviceCameraWirelessProfilesIds
    ) -> CreateOrganizationActionBatchActionsItem:
        """Assign wireless profiles to the given camera.

        [API documentation: updateDeviceCameraWirelessProfiles](https://developer.cisco.com/meraki/api-v1/#!update-device-camera-wireless-profiles)

        Args:
            serial: Serial.
            ids: The ids of the wireless profile to assign to the given camera.

        """
        serial = urllib.parse.quote(str(serial), safe="")
        path = f"/devices/{serial}/camera/wirelessProfiles"

        payload: dict[str, Any] = {}
        if ids is not None:
            payload["ids"] = ids.model_dump(by_alias=True, exclude_none=True)

        return CreateOrganizationActionBatchActionsItem(
            resource=path,
            operation="update",
            body=payload,
        )



"""A script that generates the Meraki Python library using the public OpenAPI specification."""

import json
import logging
import os
import re
import shutil
import subprocess
import sys
import textwrap
import time
import tomllib
from collections.abc import KeysView
from concurrent.futures import ThreadPoolExecutor, as_completed
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any, Literal, TextIO, TypeAlias, TypeVar, assert_never
from urllib.parse import unquote

import httpx
import jinja2
from openapi_pydantic.v3.v3_0 import (
    DataType,
    OpenAPI,
    Operation,
    Parameter,
    ParameterLocation,
    Reference,
    RequestBody,
    Schema,
)
from pydantic import BaseModel
from pydantic.alias_generators import to_snake

from codegen.constants import SCRIPT_DIR, TEMPLATE_DIR
from codegen.log_config import setup_logging
from codegen.schema_gen import (
    SchemaRegistry,
    generate_response_schemas,
    get_request_param_schema_name,
    get_response_schema_name,
)
from codegen.schemas import BatchableAction
from codegen.test_gen import generate_tests
from codegen.utils import (
    capitalize_first,
    escape_reserved_name,
    load_spec_overrides,
    sanitize_text,
    to_snake_case,
)

setup_logging()
log = logging.getLogger("codegen")

PROJECT_ROOT = Path(SCRIPT_DIR).parent
OUTPUT_DIR = "meraki_client"

REVERSE_PAGINATION = ["getNetworkEvents", "getOrganizationConfigurationChanges"]
INDENT_WIDTH = 12
DOCSTRING_LINE_WIDTH = 100 - INDENT_WIDTH

DOCS_DIR = "docs/api_reference"
MODULE_DISPLAY_TITLES: dict[str, str] = {
    "campusGateway": "Campus Gateway",
    "cellularGateway": "Cellular Gateway",
    "nac": "NAC",
    "sm": "Systems Manager",
    "wirelessController": "Wireless Controller",
}


@dataclass
class BodyParam:
    """Body parameter info."""

    snake_name: str
    orig_name: str
    is_schema: bool = False
    is_list: bool = False
    is_dict: bool = False


@dataclass
class FunctionDefinition:
    """Function definition parameters."""

    body_params: list[BodyParam] = field(default_factory=list)
    param_descriptions: list[str] = field(default_factory=list)
    required_args: list[str] = field(default_factory=list)
    optional_args: list[str] = field(default_factory=list)
    query_params: list[tuple[str, str]] = field(default_factory=list)
    path_params: list[str] = field(default_factory=list)
    request_body_schemas_used: list[str] = field(default_factory=list)


@dataclass
class ModuleInfo:
    """Module information for template generation."""

    snake_name: str
    class_name: str
    attr_name: str


@dataclass
class Templates:
    """Jinja templates."""

    api_reference_template: jinja2.Template
    batch_class_template: jinja2.Template
    batch_function_template: jinja2.Template
    batch_init_template: jinja2.Template
    class_template: jinja2.Template
    function_template: jinja2.Template
    init_template: jinja2.Template
    schema_base_template: jinja2.Template
    schema_init_template: jinja2.Template
    schema_module_template: jinja2.Template
    session_template: jinja2.Template
    types_template: jinja2.Template


PathsType: TypeAlias = dict[str, dict[Literal["get", "put", "post", "delete"], Operation]]
EnumAliasKey: TypeAlias = tuple[str, Literal["param", "body"], str]


def main() -> None:
    """Main function to generate the library."""
    t_start = time.perf_counter()

    api_version = get_api_version()
    if not api_version.startswith("v"):
        api_version = f"v{api_version}"

    client_version = get_client_version()
    log.info(f"Client version: {client_version}")
    log.info(f"API version: {api_version}")

    spec = get_openapi_specification(api_version)
    batchable_actions = [
        BatchableAction.model_validate(action) for action in spec["x-batchable-actions"]
    ]
    generate_library(OpenAPI.model_validate(spec), batchable_actions, client_version, api_version)
    elapsed = time.perf_counter() - t_start
    log.info(f"Completed code generation in {elapsed:.2f}s")


def get_api_version() -> str:
    """Read the API version from .api-version file."""
    api_version_path = PROJECT_ROOT / ".api-version"
    return api_version_path.read_text().strip()


def get_client_version() -> str:
    """Read the client version from pyproject.toml."""
    pyproject_path = PROJECT_ROOT / "pyproject.toml"
    with pyproject_path.open("rb") as f:
        pyproject = tomllib.load(f)
    return pyproject["project"]["version"]


def get_min_python_version() -> tuple[int, int]:
    """Read the minimum Python version from .python-version file."""
    python_version_path = PROJECT_ROOT / ".python-version"
    version_str = python_version_path.read_text().strip()
    parts = version_str.split(".")
    return (int(parts[0]), int(parts[1]))


def get_openapi_specification(api_version: str) -> dict[str, Any]:
    """Retrieve the OpenAPI specification from GitHub repository.

    Caches the specification locally to avoid unnecessary network requests.
    """
    spec_path = PROJECT_ROOT / ".cache" / f"spec-{api_version}.json"
    if spec_path.exists():
        log.info("Using cached OpenAPI specification")
        with spec_path.open("r") as f:
            return json.load(f)

    spec_path.parent.mkdir(parents=True, exist_ok=True)

    log.info("Downloading OpenAPI specification from GitHub repository")
    try:
        with httpx.stream(
            "GET",
            f"https://raw.githubusercontent.com/meraki/openapi/refs/tags/{api_version}/openapi/spec3.json",
        ) as response:
            response.raise_for_status()
            with spec_path.open("w") as f:
                for chunk in response.iter_bytes():
                    f.write(chunk.decode("utf-8"))
    except httpx.HTTPError as e:
        sys.exit(f"Error retrieving OpenAPI specification: {e}")

    return json.load(spec_path.open("r"))


def generate_library(  # noqa: PLR0915
    spec: OpenAPI, batchable_actions: list[BatchableAction], version_number: str, api_version: str
) -> None:
    """Generate the Meraki Python library using the public OpenAPI specification."""
    batchable_actions_map = {action.summary: action.operation for action in batchable_actions}
    spec_overrides = load_spec_overrides()

    recreate_output_directory()
    copy_static_files()

    templates = init_templates()
    min_python_version = get_min_python_version()

    t_start = time.perf_counter()
    schema_registry = generate_response_schemas(spec, templates, OUTPUT_DIR)
    elapsed = time.perf_counter() - t_start
    log.info(f"Generated {len(schema_registry.schema_names)} response schemas in {elapsed:.2f}s")

    t_start = time.perf_counter()
    scopes: dict[str, PathsType] = {}
    operation_count = 0
    for path, path_item in spec.paths.items():
        operations: dict[
            Literal["get", "put", "post", "delete", "patch", "options", "head", "trace"],
            Operation | None,
        ] = {
            "get": path_item.get,
            "put": path_item.put,
            "post": path_item.post,
            "delete": path_item.delete,
            "patch": path_item.patch,
            "options": path_item.options,
            "head": path_item.head,
            "trace": path_item.trace,
        }
        for method, operation in operations.items():
            if not operation:
                continue
            if method in ["options", "head", "trace", "patch"]:
                log.warning(f"Unsupported method: {method} for path: {path}")
                continue
            # First tag is the scope
            scope = operation.tags[0] if operation.tags else None
            if not scope:
                log.warning(f"Operation {operation.operationId} has no tags")
                continue
            scopes.setdefault(scope, {}).setdefault(path, {})[method] = operation
            operation_count += 1

    # Collect module info for __init__.py generation
    modules = sorted(
        [
            ModuleInfo(
                snake_name=to_snake_case(scope),
                class_name=capitalize_first(scope),
                attr_name=to_snake_case(scope),
            )
            for scope in scopes
        ],
        key=lambda m: m.snake_name,
    )

    with open(f"{OUTPUT_DIR}/__init__.py", "w") as f:
        f.write(
            templates.init_template.render(
                modules=modules,
                version=version_number,
                api_version=api_version,
                min_python_version=min_python_version,
                is_async=False,
            )
        )
    with open(f"{OUTPUT_DIR}/aio/__init__.py", "w") as f:
        f.write(
            templates.init_template.render(
                modules=modules,
                version=version_number,
                api_version=api_version,
                min_python_version=min_python_version,
                is_async=True,
            )
        )
    enum_type_aliases, enum_alias_lookup = collect_enum_type_aliases(spec)
    with open(f"{OUTPUT_DIR}/types.py", "w") as f:
        f.write(templates.types_template.render(enum_type_aliases=enum_type_aliases))

    with open(f"{OUTPUT_DIR}/_session.py", "w") as f:
        f.write(templates.session_template.render(is_async=False))
    with open(f"{OUTPUT_DIR}/aio/_session.py", "w") as f:
        f.write(templates.session_template.render(is_async=True))

    def generate_scope(scope: str, paths: PathsType) -> ModuleInfo | None:
        """Generate a single scope's modules.

        Returns:
            ModuleInfo if scope has batch endpoints, None otherwise.

        """
        module_name = to_snake_case(scope)
        with (
            open(f"{OUTPUT_DIR}/_api/{module_name}.py", "w") as output,
            open(f"{OUTPUT_DIR}/aio/_api/{module_name}.py", "w") as async_output,
        ):
            batch_content = generate_module(
                scope=scope,
                paths=paths,
                spec=spec,
                batchable_actions_map=batchable_actions_map,
                output=output,
                async_output=async_output,
                templates=templates,
                schema_registry=schema_registry,
                force_paginated=spec_overrides.force_paginated,
                optional_response=spec_overrides.optional_response,
                enum_alias_lookup=enum_alias_lookup,
            )

        if batch_content:
            with open(f"{OUTPUT_DIR}/_api/batch/{module_name}.py", "w") as batch_output:
                batch_output.write(batch_content)
            return ModuleInfo(
                snake_name=module_name,
                class_name=capitalize_first(scope),
                attr_name=module_name,
            )
        return None

    batch_modules: list[ModuleInfo] = []
    with ThreadPoolExecutor() as executor:
        futures = {
            executor.submit(generate_scope, scope, paths): scope for scope, paths in scopes.items()
        }
        for future in as_completed(futures):
            scope = futures[future]
            try:
                batch_module = future.result()
                if batch_module:
                    batch_modules.append(batch_module)
            except Exception:
                log.exception(f"Failed to generate {scope}")
                raise
            log.debug(f"Generated {scope}")

    batch_modules.sort(key=lambda m: m.snake_name)
    with open(f"{OUTPUT_DIR}/_api/batch/__init__.py", "w") as f:
        f.write(templates.batch_init_template.render(modules=batch_modules))
    elapsed = time.perf_counter() - t_start
    log.info(f"Generated {len(scopes)} modules and {operation_count} operations in {elapsed:.2f}s")

    t_start = time.perf_counter()
    _format_generated_code(OUTPUT_DIR)
    elapsed = time.perf_counter() - t_start
    log.info(f"Formatted generated code in {elapsed:.2f}s")

    t_start = time.perf_counter()
    generate_tests(spec, skip_tests=spec_overrides.skip_tests)
    elapsed = time.perf_counter() - t_start
    log.info(f"Generated tests in {elapsed:.2f}s")

    t_start = time.perf_counter()
    generate_api_reference_docs(scopes.keys(), batch_modules, templates)
    elapsed = time.perf_counter() - t_start
    log.info(f"Generated API reference docs in {elapsed:.2f}s")


def generate_module(  # noqa: PLR0915
    *,
    scope: str,
    paths: PathsType,
    spec: OpenAPI,
    batchable_actions_map: dict[str, str],
    output: TextIO,
    async_output: TextIO,
    templates: Templates,
    schema_registry: SchemaRegistry,
    force_paginated: set[str],
    optional_response: set[str],
    enum_alias_lookup: dict[EnumAliasKey, str],
) -> str | None:
    """Generate a module for a scope.

    Returns:
        Batch module content if scope has batch endpoints, None otherwise.

    """
    class_name = capitalize_first(scope)
    schemas_used: list[str] = []
    batch_content: list[str] = []
    batch_schemas_used: list[str] = []
    operation_request_schemas: dict[str, list[str]] = {}
    enum_type_aliases_used: set[str] = set()
    batch_enum_type_aliases_used: set[str] = set()

    for methods in paths.values():
        for method, endpoint in methods.items():
            operation_id = endpoint.operationId
            if not operation_id:
                continue

            # Collect request body schemas for this operation
            op_schemas: list[str] = []
            for (op_id, _prop_name), param_schema in schema_registry.request_body_schemas.items():
                if op_id == operation_id:
                    if param_schema.item_class_name:
                        op_schemas.append(param_schema.item_class_name)
                    elif not param_schema.is_list:
                        op_schemas.append(param_schema.class_name)
            if op_schemas:
                operation_request_schemas[operation_id] = op_schemas
                schemas_used.extend(op_schemas)

            operation_enum_aliases = collect_operation_enum_type_aliases(
                spec=spec,
                operation_id=operation_id,
                endpoint=endpoint,
                enum_alias_lookup=enum_alias_lookup,
            )
            enum_type_aliases_used.update(operation_enum_aliases)
            if batchable_actions_map.get(endpoint.description or ""):
                batch_enum_type_aliases_used.update(operation_enum_aliases)

            if method == "delete":
                continue

            response_schema_name = get_response_schema_name(operation_id)
            if response_schema_name in schema_registry.schema_names:
                schemas_used.append(response_schema_name)
            if response_schema_name in schema_registry.item_schema_map:
                schemas_used.extend(schema_registry.item_schema_map[response_schema_name])

    output.write(
        templates.class_template.render(
            class_name=class_name,
            is_async=False,
            schemas_used=sorted(set(schemas_used)),
            enum_type_aliases=sorted(enum_type_aliases_used),
        )
    )
    async_output.write(
        templates.class_template.render(
            class_name=class_name,
            is_async=True,
            schemas_used=sorted(set(schemas_used)),
            enum_type_aliases=sorted(enum_type_aliases_used),
        )
    )

    for path, methods in paths.items():
        for method, endpoint in methods.items():
            operation_id = endpoint.operationId
            if not operation_id:
                log.warning(f"Operation ID is missing for path: {path}")
                continue

            description = sanitize_text(str(endpoint.summary))
            resource_path = convert_path_params(path)
            function_definition = FunctionDefinition()
            is_paginated = collect_params(
                spec,
                operation_id,
                endpoint.parameters or [],
                function_definition,
                enum_alias_lookup,
            )
            has_pagination_params = is_paginated
            collect_request_body_params(
                spec,
                operation_id,
                endpoint.requestBody,
                function_definition,
                schema_registry,
                enum_alias_lookup,
            )
            if is_paginated:
                collect_pagination_params(operation_id, function_definition)
            is_paginated = check_force_paginated(
                operation_id=operation_id,
                is_paginated=is_paginated,
                force_paginated=force_paginated,
            )

            response_schema_name = None
            item_schema_name = None
            has_untyped_response = operation_id in schema_registry.untyped_response_ops
            if has_untyped_response:
                response_schema_name = "DictResponse"
            if method != "delete":
                schema_name = get_response_schema_name(operation_id)
                # GET endpoints with wrapper shape {items, meta} are generated as paginated item
                # schemas only, so treat them as paginated responses.
                if (
                    method == "get"
                    and schema_name not in schema_registry.schema_names
                    and schema_name in schema_registry.item_schema_map
                ):
                    is_paginated = True

                if schema_name in schema_registry.schema_names:
                    response_schema_name = schema_name
                # For paginated endpoints, use first item schema from map or fallback to response schema.
                # (some OpenAPI specs incorrectly define array responses as objects)
                if is_paginated:
                    item_schemas = schema_registry.item_schema_map.get(schema_name)
                    if item_schemas:
                        item_schema_name = item_schemas[0]
                    elif response_schema_name:
                        item_schema_name = response_schema_name

            is_list_response = (
                response_schema_name is not None
                and response_schema_name in schema_registry.list_response_schemas
            )
            if is_list_response and not is_paginated and method == "get":
                is_paginated = True
                # Get item schema for list response
                assert response_schema_name is not None
                item_schemas = schema_registry.item_schema_map.get(response_schema_name)
                item_schema_name = item_schemas[0] if item_schemas else response_schema_name

            definition = build_function_definition(
                function_definition.required_args, function_definition.optional_args
            )

            if is_paginated and not item_schema_name:
                raise ValueError(f"Paginated endpoint {operation_id} has no response schema")

            if method != "delete" and not response_schema_name and not has_untyped_response:
                responses = endpoint.responses or {}
                if ("200" in responses or "201" in responses) and not (
                    is_paginated and item_schema_name
                ):
                    raise ValueError(f"Endpoint {operation_id} has 200/201 response but no schema")

            body_params_for_template = [
                (bp.snake_name, bp.orig_name, bp.is_schema, bp.is_list, bp.is_dict)
                for bp in function_definition.body_params
            ]

            return_description, response_example = get_response_info(endpoint)
            use_raw_docstring = response_example is not None and "\\" in response_example

            common_params = {
                "function_name": to_snake_case(operation_id),
                "operation_id": operation_id,
                "function_definition": definition,
                "description": description,
                "doc_url": docs_url(operation_id),
                "descriptions": function_definition.param_descriptions,
                "resource": resource_path,
                "query_params": function_definition.query_params,
                "body_params": body_params_for_template,
                "path_params": function_definition.path_params,
                "response_schema_name": response_schema_name,
                "item_schema_name": item_schema_name,
                "return_description": return_description,
                "response_example": response_example,
                "use_raw_docstring": use_raw_docstring,
            }

            is_optional_response = operation_id in optional_response
            output.write(
                templates.function_template.render(
                    **common_params,
                    method=method,
                    scope=scope,
                    return_type=get_return_type(
                        method=method,
                        is_paginated=is_paginated,
                        is_async=False,
                        response_schema_name=response_schema_name,
                        item_schema_name=item_schema_name,
                        optional_response=is_optional_response,
                    ),
                    is_async=False,
                    is_paginated=is_paginated,
                    is_optional_response=is_optional_response,
                    has_pagination_params=has_pagination_params,
                )
            )
            async_output.write(
                templates.function_template.render(
                    **common_params,
                    method=method,
                    scope=scope,
                    return_type=get_return_type(
                        method=method,
                        is_paginated=is_paginated,
                        is_async=True,
                        response_schema_name=response_schema_name,
                        item_schema_name=item_schema_name,
                        optional_response=is_optional_response,
                    ),
                    is_async=True,
                    is_paginated=is_paginated,
                    is_optional_response=is_optional_response,
                    has_pagination_params=has_pagination_params,
                )
            )

            batch_operation = batchable_actions_map.get(endpoint.description or "")
            if batch_operation:
                batch_schemas_used.extend(function_definition.request_body_schemas_used)

                batch_content.append(
                    templates.batch_function_template.render(
                        operation_id=operation_id,
                        function_name=to_snake_case(operation_id),
                        function_definition=definition,
                        description=description,
                        doc_url=docs_url(operation_id),
                        descriptions=function_definition.param_descriptions,
                        resource=resource_path,
                        query_params=function_definition.query_params,
                        body_params=body_params_for_template,
                        path_params=function_definition.path_params,
                        batch_operation=batch_operation,
                    )
                )

    if batch_content:
        return templates.batch_class_template.render(
            class_name=class_name,
            schemas_used=sorted(set(batch_schemas_used)),
            enum_type_aliases=sorted(batch_enum_type_aliases_used),
        ) + "".join(batch_content)
    return None


def generate_api_reference_docs(
    scopes: list[str] | KeysView[str],
    batch_modules: list[ModuleInfo],
    templates: Templates,
) -> None:
    """Generate API reference markdown docs for mkdocs."""
    os.makedirs(DOCS_DIR, exist_ok=True)

    # Generate module docs
    for scope in scopes:
        module_name = to_snake_case(scope)
        title = MODULE_DISPLAY_TITLES.get(scope, capitalize_first(scope))
        content = templates.api_reference_template.render(
            title=title,
            module_path=f"meraki_client._api.{module_name}.{capitalize_first(scope)}",
        )
        with open(f"{DOCS_DIR}/{module_name}.md", "w") as f:
            f.write(content)

    # Generate batch module docs
    for batch_module in batch_modules:
        title = MODULE_DISPLAY_TITLES.get(
            batch_module.class_name, capitalize_first(batch_module.class_name)
        )
        content = templates.api_reference_template.render(
            title=f"{title} (Batch)",
            module_path=f"meraki_client._api.batch.{batch_module.snake_name}.ActionBatch{batch_module.class_name}",
        )
        with open(f"{DOCS_DIR}/batch_{batch_module.snake_name}.md", "w") as f:
            f.write(content)


def build_function_definition(required_args: list[str], optional_args: list[str]) -> str:
    """Build function definition string with appropriate positional/keyword-only parameters.

    If there's exactly one required parameter, it's positional. All other parameters
    (including optional and multiple required) are keyword-only.
    """
    if not required_args and not optional_args:
        return ""

    if len(required_args) == 1:
        # Single required param is positional, optional params are keyword-only
        if optional_args:
            return f", {required_args[0]}, *, " + ", ".join(optional_args)
        return f", {required_args[0]}"

    # Multiple required or no required: all keyword-only
    all_args = required_args + optional_args
    return ", *, " + ", ".join(all_args)


def recreate_output_directory() -> None:
    """Recreate the output directory."""
    if os.path.exists(OUTPUT_DIR):
        shutil.rmtree(OUTPUT_DIR)

    subdirs = [
        OUTPUT_DIR,
        f"{OUTPUT_DIR}/_api",
        f"{OUTPUT_DIR}/_api/batch",
        f"{OUTPUT_DIR}/aio",
        f"{OUTPUT_DIR}/aio/_api",
        f"{OUTPUT_DIR}/schemas",
    ]
    for directory in subdirs:
        os.makedirs(directory, exist_ok=True)


def copy_static_files() -> None:
    """Copy static files from the static directory to the output directory."""
    static_dir = Path(SCRIPT_DIR) / "static"
    for src in static_dir.rglob("*"):
        if src.is_file():
            relative_path = src.relative_to(static_dir)
            dst = Path(OUTPUT_DIR) / relative_path
            dst.parent.mkdir(parents=True, exist_ok=True)
            shutil.copy2(src, dst)


def _format_generated_code(output_dir: str) -> None:
    subprocess.run(  # noqa: S603
        ["uv", "run", "ruff", "check", "--quiet", "--select", "I,F401,RUF022", "--fix", output_dir],  # noqa: S607
        check=True,
    )
    subprocess.run(  # noqa: S603
        ["uv", "run", "ruff", "format", "--quiet", output_dir],  # noqa: S607
        check=True,
    )


def init_templates() -> Templates:
    """Initialize the templates."""
    jinja_env = jinja2.Environment(trim_blocks=True, lstrip_blocks=True, keep_trailing_newline=True)  # noqa: S701
    return Templates(
        api_reference_template=read_template("api_reference_template.md", jinja_env),
        batch_class_template=read_template("batch_class_template.py", jinja_env),
        batch_function_template=read_template("batch_function_template.py", jinja_env),
        batch_init_template=read_template("batch_init_template.py", jinja_env),
        class_template=read_template("class_template.py", jinja_env),
        function_template=read_template("function_template.py", jinja_env),
        init_template=read_template("init_template.py", jinja_env),
        schema_base_template=read_template("schema_base_template.py", jinja_env),
        schema_init_template=read_template("schema_init_template.py", jinja_env),
        schema_module_template=read_template("schema_module_template.py", jinja_env),
        session_template=read_template("session_template.py", jinja_env),
        types_template=read_template("types_template.py", jinja_env),
    )


def read_template(template_name: str, jinja_env: jinja2.Environment) -> jinja2.Template:
    """Read a template from the template directory."""
    with open(
        os.path.join(TEMPLATE_DIR, f"{template_name}.jinja2"), encoding="utf-8", newline=None
    ) as fp:
        return jinja_env.from_string(fp.read())


def collect_params(
    spec: OpenAPI,
    operation_id: str,
    parameters: list[Parameter | Reference],
    function_definition: FunctionDefinition,
    enum_alias_lookup: dict[EnumAliasKey, str],
) -> bool:
    """Collect path and query parameters. Returns True if paginated."""
    is_paginated = False
    for param in parameters:
        if isinstance(param, Reference):
            param = resolve_ref(spec, param, Parameter)  # noqa: PLW2901
            if not param:
                log.warning(f"Failed to resolve parameter reference: {param}")
                continue

        param_name = param.name
        snake_name = escape_reserved_name(to_snake_case(param_name))

        param_schema = param.param_schema
        if not param_schema:
            log.warning(f"No schema found for parameter: {param_name}")
            continue
        if isinstance(param_schema, Reference):
            param_schema = resolve_ref(spec, param_schema, Schema)
            if not param_schema:
                log.error(f"Failed to resolve schema reference: {param_schema}")
                continue

        py_type = get_python_type(param_schema)
        alias_name = enum_alias_lookup.get((operation_id, "param", param_name))
        if alias_name:
            py_type = alias_name

        if param.param_in == ParameterLocation.QUERY:
            key = param_name
            if param_schema.type == DataType.ARRAY:
                # Query parameter syntax for arrays is param[]=value
                key += "[]"
            function_definition.query_params.append((snake_name, key))
        elif param.param_in == ParameterLocation.PATH:
            function_definition.path_params.append(snake_name)
        else:
            log.error(
                f"Unsupported parameter location '{param.param_in}' for operation: {operation_id}"
            )
            continue

        function_definition.param_descriptions.append(
            format_param_description(
                snake_name, param.description or param_schema.description or ""
            )
        )

        if snake_name == "per_page":
            is_paginated = True

        if param.required or param.param_in == ParameterLocation.PATH:
            function_definition.required_args.append(f"{snake_name}: {py_type}")
        else:
            function_definition.optional_args.append(f"{snake_name}: {py_type} | None = None")

    return is_paginated


def collect_request_body_params(
    spec: OpenAPI,
    operation_id: str,
    request_body: RequestBody | Reference | None,
    function_definition: FunctionDefinition,
    schema_registry: SchemaRegistry,
    enum_alias_lookup: dict[EnumAliasKey, str],
) -> None:
    """Collect request body parameters."""
    if isinstance(request_body, Reference):
        request_body = resolve_ref(spec, request_body, RequestBody)

    if not request_body:
        return

    json_content = request_body.content.get("application/json")
    if not json_content:
        log.warning(f"No JSON content found in request body for operation: {operation_id}")
        return

    content_schema = json_content.media_type_schema
    if isinstance(content_schema, Reference):
        content_schema = resolve_ref(spec, content_schema, Schema)
    if not content_schema:
        log.warning(f"No schema found in request body for operation: {operation_id}")
        return

    properties = content_schema.properties or {}
    for property_name, property_schema in properties.items():
        snake_name = escape_reserved_name(to_snake_case(property_name))
        if isinstance(property_schema, Reference):
            property_schema = resolve_ref(spec, property_schema, Schema)  # noqa: PLW2901
            if not property_schema:
                log.error(f"Failed to resolve schema reference: {property_schema}")
                continue

        schema_key = (operation_id, property_name)
        param_schema = schema_registry.request_body_schemas.get(schema_key)

        is_schema = param_schema is not None
        is_list = param_schema.is_list if param_schema else False
        is_dict = param_schema.is_dict if param_schema else False
        function_definition.body_params.append(
            BodyParam(snake_name, property_name, is_schema, is_list, is_dict)
        )

        if (
            property_name == "scheduleId"
            and operation_id == "deleteOrganizationDevicesPacketCaptureSchedule"
        ):
            # Schedule ID is duplicate of path param in this operation
            continue

        function_definition.param_descriptions.append(
            format_param_description(snake_name, property_schema.description or "")
        )

        if param_schema:
            py_type = param_schema.class_name
            if param_schema.item_class_name:
                function_definition.request_body_schemas_used.append(param_schema.item_class_name)
            elif not param_schema.is_list:
                function_definition.request_body_schemas_used.append(param_schema.class_name)
        else:
            py_type = get_python_type(property_schema)

        alias_name = enum_alias_lookup.get((operation_id, "body", property_name))
        if alias_name:
            py_type = alias_name

        required_properties = content_schema.required or []
        if property_name in required_properties:
            function_definition.required_args.append(f"{snake_name}: {py_type}")
        else:
            function_definition.optional_args.append(f"{snake_name}: {py_type} | None = None")


def collect_operation_enum_type_aliases(
    *,
    spec: OpenAPI,
    operation_id: str,
    endpoint: Operation,
    enum_alias_lookup: dict[EnumAliasKey, str],
) -> set[str]:
    """Collect enum type aliases used by a single endpoint."""
    aliases: set[str] = set()

    for param in endpoint.parameters or []:
        resolved_param = param
        if isinstance(resolved_param, Reference):
            resolved_param = resolve_ref(spec, resolved_param, Parameter)
            if not resolved_param:
                continue
        alias_name = enum_alias_lookup.get((operation_id, "param", resolved_param.name))
        if alias_name:
            aliases.add(alias_name)

    request_body = endpoint.requestBody
    if isinstance(request_body, Reference):
        request_body = resolve_ref(spec, request_body, RequestBody)
    if not request_body:
        return aliases

    json_content = request_body.content.get("application/json")
    if not json_content:
        return aliases

    content_schema = json_content.media_type_schema
    if isinstance(content_schema, Reference):
        content_schema = resolve_ref(spec, content_schema, Schema)
    if not content_schema:
        return aliases

    for property_name in content_schema.properties or {}:
        alias_name = enum_alias_lookup.get((operation_id, "body", property_name))
        if alias_name:
            aliases.add(alias_name)

    return aliases


def check_force_paginated(
    *,
    operation_id: str,
    is_paginated: bool,
    force_paginated: set[str],
) -> bool:
    """Check if endpoint should be force-paginated due to spec bugs.

    Returns True if endpoint should be treated as paginated.
    Logs warning if endpoint appears to be fixed in spec.
    """
    if operation_id not in force_paginated:
        return is_paginated

    if is_paginated:
        log.warning(
            f"{operation_id} in force_paginated now has per_page param - "
            "spec may be fixed, check if override still needed"
        )
    return True


def collect_pagination_params(operation_id: str, function_definition: FunctionDefinition) -> None:
    """Collect pagination parameters."""
    function_definition.optional_args.append('total_pages: int | Literal["all"] = "all"')

    function_definition.param_descriptions.append(
        format_param_description(
            "total_pages",
            "use with per_page to get total results"
            ' up to total_pages * per_page; -1 or "all" for all pages',
        )
    )

    if operation_id in REVERSE_PAGINATION:
        function_definition.optional_args.append("direction: Literal['prev', 'next'] = 'prev'")
    else:
        function_definition.optional_args.append("direction: Literal['prev', 'next'] = 'next'")

    function_definition.param_descriptions.append(
        format_param_description(
            "direction",
            'direction to paginate, either "next" or "prev" (default) page'
            if operation_id in REVERSE_PAGINATION
            else 'direction to paginate, either "next" (default) or "prev" page',
        )
    )

    if operation_id == "getNetworkEvents":
        function_definition.optional_args.append("event_log_end_time: str | None = None")
        function_definition.param_descriptions.append(
            format_param_description(
                "event_log_end_time",
                "ISO8601 Zulu/UTC time, to use in conjunction with starting_after, "
                "to retrieve events within a time window",
            )
        )


def get_return_type(
    *,
    method: Literal["get", "put", "post", "delete"],
    is_paginated: bool,
    is_async: bool,
    response_schema_name: str | None = None,
    item_schema_name: str | None = None,
    optional_response: bool = False,
) -> str:
    """Get the return type for a function."""
    if method == "delete":
        return "None"
    if method == "get" and is_paginated and item_schema_name:
        return (
            f"AsyncPaginatedResponse[{item_schema_name}]"
            if is_async
            else f"PaginatedResponse[{item_schema_name}]"
        )
    if response_schema_name:
        if optional_response:
            return f"{response_schema_name} | None"
        return f"{response_schema_name}"
    return "None"


def get_response_info(endpoint: Operation) -> tuple[str | None, str | None]:
    """Get the return description and example from the OpenAPI response."""
    responses = endpoint.responses or {}
    for status_code, response in responses.items():
        if not status_code.startswith("2"):
            continue
        if isinstance(response, Reference):
            continue

        description = sanitize_text(response.description) if response.description else None

        example = None
        content = response.content
        if content:
            json_content = content.get("application/json")
            if json_content and json_content.example:
                example = json.dumps(json_content.example, indent=2)

        return description, example

    return None, None


def format_param_description(name: str, description: str) -> str:
    """Format a parameter description for Google-style docstring with line wrapping."""
    first_line = f"{name}: {sanitize_text(description)}"
    if len(first_line) <= DOCSTRING_LINE_WIDTH:
        return first_line

    wrapper = textwrap.TextWrapper(
        width=DOCSTRING_LINE_WIDTH,
        initial_indent="",
        subsequent_indent=" " * (INDENT_WIDTH + 4),
    )
    return wrapper.fill(first_line)


_T = TypeVar("_T")


def resolve_ref(
    spec: OpenAPI,
    ref: Reference,
    expected_type: type[_T],
    _seen: set[str] | None = None,
) -> _T | None:
    """Resolve a $ref reference in OASv3 spec."""
    ref_str = ref.ref
    if not ref_str.startswith("#/"):
        # Ignore external refs
        return None

    if _seen is None:
        _seen = set()
    if ref_str in _seen:
        return None
    _seen = _seen | {ref_str}

    parts = [unquote(p) for p in ref_str[2:].split("/")]

    result: Any = spec
    for part in parts:
        if isinstance(result, dict):
            if part in result:
                result = result[part]
            else:
                return None
        elif isinstance(result, BaseModel):
            if hasattr(result, part):
                result = getattr(result, part)
            else:
                found = False
                for field_name, field_info in type(result).model_fields.items():
                    if field_info.alias == part:
                        result = getattr(result, field_name)
                        found = True
                        break
                if not found:
                    return None
        else:
            return None

    if isinstance(result, Reference):
        return resolve_ref(spec, result, expected_type, _seen)

    if isinstance(result, expected_type):
        return result
    return None


def docs_url(operation_id: str) -> str:
    """Returns full link to endpoint's documentation on Developer Hub."""
    base_url = "https://developer.cisco.com/meraki/api-v1/#!"
    kebab = to_snake(operation_id).replace("_", "-")
    # Insert hyphen between letter and digit: l3 -> l-3, hotspot20 -> hotspot-20
    kebab = re.sub(r"([a-z])(\d)", r"\1-\2", kebab)
    return base_url + kebab


def convert_path_params(path: str) -> str:
    """Convert all {paramName} in path to {param_name} (snake_case, sanitized)."""

    def replace_param(match: re.Match[str]) -> str:
        param = match.group(1)
        return f"{{{escape_reserved_name(to_snake_case(param))}}}"

    return re.sub(r"\{(\w+)\}", replace_param, path)


def format_literal_type(enum_values: list[Any]) -> str:
    """Format enum values as a Literal type."""
    if not enum_values:
        raise ValueError("Enum values are required")
    return f"Literal[{', '.join(repr(value) for value in enum_values)}]"


def build_enum_type_alias_name(operation_id: str, param_name: str) -> str:
    """Build a stable alias name for an enum request parameter."""
    return get_request_param_schema_name(operation_id, param_name)


def register_enum_type_alias(
    enum_type_aliases: dict[str, str], alias_name: str, alias_type: str
) -> str:
    """Register enum type alias while handling rare name collisions."""
    if alias_name not in enum_type_aliases:
        enum_type_aliases[alias_name] = alias_type
        return alias_name

    if enum_type_aliases[alias_name] == alias_type:
        return alias_name

    suffix = 2
    while True:
        candidate = f"{alias_name}{suffix}"
        if candidate not in enum_type_aliases:
            log.warning(
                f"Enum alias collision on {alias_name}; using {candidate} for differing type"
            )
            enum_type_aliases[candidate] = alias_type
            return candidate
        if enum_type_aliases[candidate] == alias_type:
            return candidate
        suffix += 1


def get_python_type(schema: Schema) -> str:
    """Get Python type for a schema."""
    if schema.enum:
        return format_literal_type(schema.enum)

    data_type = schema.type or DataType.STRING
    match data_type:
        case DataType.ARRAY:
            items = schema.items
            if items and isinstance(items, Schema) and items.type:
                item_type = get_python_type(items)
                return f"list[{item_type}]"
            log.warning(f"Unknown array items type: {items}")
            return "list[Any]"
        case DataType.NUMBER:
            return "float"
        case DataType.INTEGER:
            return "int"
        case DataType.BOOLEAN:
            return "bool"
        case DataType.OBJECT:
            return "dict[str, Any]"
        case DataType.STRING:
            return "str"
        case _:
            assert_never(data_type)


def collect_param_enum_type_aliases(
    *,
    spec: OpenAPI,
    operation_id: str,
    endpoint: Operation,
    enum_type_aliases: dict[str, str],
    enum_alias_lookup: dict[EnumAliasKey, str],
) -> None:
    """Collect enum aliases from operation parameters."""
    for param in endpoint.parameters or []:
        resolved_param = param
        if isinstance(resolved_param, Reference):
            resolved_param = resolve_ref(spec, resolved_param, Parameter)
            if not resolved_param:
                continue

        param_schema = resolved_param.param_schema
        if not param_schema:
            continue
        if isinstance(param_schema, Reference):
            param_schema = resolve_ref(spec, param_schema, Schema)
            if not param_schema:
                continue

        py_type = get_python_type(param_schema)
        if "Literal[" not in py_type:
            continue

        alias_name = build_enum_type_alias_name(operation_id, resolved_param.name)
        resolved_alias_name = register_enum_type_alias(enum_type_aliases, alias_name, py_type)
        enum_alias_lookup[(operation_id, "param", resolved_param.name)] = resolved_alias_name


def collect_body_enum_type_aliases(
    *,
    spec: OpenAPI,
    operation_id: str,
    endpoint: Operation,
    enum_type_aliases: dict[str, str],
    enum_alias_lookup: dict[EnumAliasKey, str],
) -> None:
    """Collect enum aliases from operation request body properties."""
    request_body = endpoint.requestBody
    if isinstance(request_body, Reference):
        request_body = resolve_ref(spec, request_body, RequestBody)
    if not request_body:
        return

    json_content = request_body.content.get("application/json")
    if not json_content:
        return

    content_schema = json_content.media_type_schema
    if isinstance(content_schema, Reference):
        content_schema = resolve_ref(spec, content_schema, Schema)
    if not content_schema:
        return

    for property_name, property_schema in (content_schema.properties or {}).items():
        resolved_property_schema = property_schema
        if isinstance(resolved_property_schema, Reference):
            resolved_property_schema = resolve_ref(spec, resolved_property_schema, Schema)
            if not resolved_property_schema:
                continue

        py_type = get_python_type(resolved_property_schema)
        if "Literal[" not in py_type:
            continue

        alias_name = build_enum_type_alias_name(operation_id, property_name)
        resolved_alias_name = register_enum_type_alias(enum_type_aliases, alias_name, py_type)
        enum_alias_lookup[(operation_id, "body", property_name)] = resolved_alias_name


def collect_enum_type_aliases(spec: OpenAPI) -> tuple[dict[str, str], dict[EnumAliasKey, str]]:
    """Collect importable enum request parameter type aliases and lookup keys."""
    enum_type_aliases: dict[str, str] = {}
    enum_alias_lookup: dict[EnumAliasKey, str] = {}

    for path_item in spec.paths.values():
        operations = [path_item.get, path_item.put, path_item.post, path_item.delete]
        for endpoint in operations:
            if not endpoint:
                continue
            operation_id = endpoint.operationId
            if not operation_id:
                continue

            collect_param_enum_type_aliases(
                spec=spec,
                operation_id=operation_id,
                endpoint=endpoint,
                enum_type_aliases=enum_type_aliases,
                enum_alias_lookup=enum_alias_lookup,
            )
            collect_body_enum_type_aliases(
                spec=spec,
                operation_id=operation_id,
                endpoint=endpoint,
                enum_type_aliases=enum_type_aliases,
                enum_alias_lookup=enum_alias_lookup,
            )

    return dict(sorted(enum_type_aliases.items())), enum_alias_lookup


if __name__ == "__main__":
    main()



"""Tests for wireless API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

from meraki_client import MerakiClient
from tests.generated.conftest import skip_on_unsupported


def test_get_device_wireless_bluetooth_settings(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_wireless_bluetooth_settings endpoint."""
    with skip_on_unsupported():
        client.wireless.get_device_wireless_bluetooth_settings(serial=device_serial)


def test_get_device_wireless_electronic_shelf_label(
    client: MerakiClient, device_serial: str
) -> None:
    """Test get_device_wireless_electronic_shelf_label endpoint."""
    with skip_on_unsupported():
        client.wireless.get_device_wireless_electronic_shelf_label(serial=device_serial)


def test_get_device_wireless_radio_settings(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_wireless_radio_settings endpoint."""
    with skip_on_unsupported():
        client.wireless.get_device_wireless_radio_settings(serial=device_serial)


def test_get_device_wireless_status(client: MerakiClient, device_serial: str) -> None:
    """Test get_device_wireless_status endpoint."""
    with skip_on_unsupported():
        client.wireless.get_device_wireless_status(serial=device_serial)


def test_get_network_wireless_air_marshal(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_air_marshal endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_air_marshal(network_id=network_id)


def test_get_network_wireless_alternate_management_interface(
    client: MerakiClient, network_id: str
) -> None:
    """Test get_network_wireless_alternate_management_interface endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_alternate_management_interface(network_id=network_id)


def test_get_network_wireless_billing(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_billing endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_billing(network_id=network_id)


def test_get_network_wireless_bluetooth_settings(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_bluetooth_settings endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_bluetooth_settings(network_id=network_id)


def test_get_network_wireless_client_count_history(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_client_count_history endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_client_count_history(network_id=network_id)


def test_get_network_wireless_data_rate_history(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_data_rate_history endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_data_rate_history(network_id=network_id)


def test_get_network_wireless_electronic_shelf_label(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_electronic_shelf_label endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_electronic_shelf_label(network_id=network_id)


def test_get_network_wireless_electronic_shelf_label_configured_devices(
    client: MerakiClient, network_id: str
) -> None:
    """Test get_network_wireless_electronic_shelf_label_configured_devices endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_electronic_shelf_label_configured_devices(
            network_id=network_id
        )


def test_get_network_wireless_ethernet_ports_profiles(
    client: MerakiClient, network_id: str
) -> None:
    """Test get_network_wireless_ethernet_ports_profiles endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_ethernet_ports_profiles(network_id=network_id)


def test_get_network_wireless_latency_history(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_latency_history endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_latency_history(network_id=network_id)


def test_get_network_wireless_mesh_statuses(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_mesh_statuses endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_network_wireless_mesh_statuses(network_id=network_id).collect()
    assert isinstance(result, list)


def test_get_network_wireless_rf_profiles(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_rf_profiles endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_rf_profiles(network_id=network_id)


def test_get_network_wireless_settings(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_settings endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_settings(network_id=network_id)


def test_get_network_wireless_ssids(client: MerakiClient, network_id: str) -> None:
    """Test get_network_wireless_ssids endpoint."""
    with skip_on_unsupported():
        client.wireless.get_network_wireless_ssids(network_id=network_id)


def test_get_organization_wireless_air_marshal_rules(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_air_marshal_rules endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_air_marshal_rules(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_air_marshal_settings_by_network(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_air_marshal_settings_by_network endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_air_marshal_settings_by_network(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_clients_overview_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_clients_overview_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_clients_overview_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_channel_utilization_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_channel_utilization_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_channel_utilization_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_channel_utilization_by_network(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_channel_utilization_by_network endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_channel_utilization_by_network(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_channel_utilization_history_by_device_by_interval(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_channel_utilization_history_by_device_by_interval endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_channel_utilization_history_by_device_by_interval(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_channel_utilization_history_by_network_by_interval(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_channel_utilization_history_by_network_by_interval endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_channel_utilization_history_by_network_by_interval(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_ethernet_statuses(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_ethernet_statuses endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_ethernet_statuses(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_packet_loss_by_client(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_packet_loss_by_client endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_packet_loss_by_client(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_packet_loss_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_packet_loss_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_packet_loss_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_packet_loss_by_network(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_packet_loss_by_network endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_packet_loss_by_network(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_power_mode_history(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_power_mode_history endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_power_mode_history(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_radsec_certificates_authorities(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_radsec_certificates_authorities endpoint."""
    with skip_on_unsupported():
        client.wireless.get_organization_wireless_devices_radsec_certificates_authorities(
            organization_id=organization_id
        )


def test_get_organization_wireless_devices_radsec_certificates_authorities_crls(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_radsec_certificates_authorities_crls endpoint."""
    with skip_on_unsupported():
        client.wireless.get_organization_wireless_devices_radsec_certificates_authorities_crls(
            organization_id=organization_id
        )


def test_get_organization_wireless_devices_radsec_certificates_authorities_crls_deltas(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_radsec_certificates_authorities_crls_deltas endpoint."""
    with skip_on_unsupported():
        client.wireless.get_organization_wireless_devices_radsec_certificates_authorities_crls_deltas(
            organization_id=organization_id
        )


def test_get_organization_wireless_devices_system_cpu_load_history(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_system_cpu_load_history endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_system_cpu_load_history(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_devices_wireless_controllers_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_devices_wireless_controllers_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_devices_wireless_controllers_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_location_scanning_by_network(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_location_scanning_by_network endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_location_scanning_by_network(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_location_scanning_receivers(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_location_scanning_receivers endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_location_scanning_receivers(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_mqtt_settings(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_mqtt_settings endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_mqtt_settings(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_radio_rrm_by_network(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_radio_rrm_by_network endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_radio_rrm_by_network(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_rf_profiles_assignments_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_rf_profiles_assignments_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_rf_profiles_assignments_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_ssids_firewall_isolation_allowlist_entries(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_ssids_firewall_isolation_allowlist_entries endpoint."""
    with skip_on_unsupported():
        result = (
            client.wireless.get_organization_wireless_ssids_firewall_isolation_allowlist_entries(
                organization_id=organization_id
            ).collect()
        )
    assert isinstance(result, list)


def test_get_organization_wireless_ssids_open_roaming_by_network(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_ssids_open_roaming_by_network endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_ssids_open_roaming_by_network(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_ssids_statuses_by_device(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_ssids_statuses_by_device endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_ssids_statuses_by_device(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_zigbee_by_network(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_zigbee_by_network endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_zigbee_by_network(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_zigbee_devices(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_zigbee_devices endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_zigbee_devices(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_wireless_zigbee_door_locks(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_wireless_zigbee_door_locks endpoint."""
    with skip_on_unsupported():
        result = client.wireless.get_organization_wireless_zigbee_door_locks(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)



"""Tests for sm API endpoints.

This file is automatically @generated from the Meraki API specification.
Do not edit this file manually.

"""

from meraki_client import MerakiClient
from tests.generated.conftest import skip_on_unsupported


def test_get_network_sm_devices(client: MerakiClient, network_id: str) -> None:
    """Test get_network_sm_devices endpoint."""
    with skip_on_unsupported():
        result = client.sm.get_network_sm_devices(network_id=network_id).collect()
    assert isinstance(result, list)


def test_get_network_sm_profiles(client: MerakiClient, network_id: str) -> None:
    """Test get_network_sm_profiles endpoint."""
    with skip_on_unsupported():
        client.sm.get_network_sm_profiles(network_id=network_id)


def test_get_network_sm_target_groups(client: MerakiClient, network_id: str) -> None:
    """Test get_network_sm_target_groups endpoint."""
    with skip_on_unsupported():
        client.sm.get_network_sm_target_groups(network_id=network_id)


def test_get_network_sm_trusted_access_configs(client: MerakiClient, network_id: str) -> None:
    """Test get_network_sm_trusted_access_configs endpoint."""
    with skip_on_unsupported():
        result = client.sm.get_network_sm_trusted_access_configs(network_id=network_id).collect()
    assert isinstance(result, list)


def test_get_network_sm_user_access_devices(client: MerakiClient, network_id: str) -> None:
    """Test get_network_sm_user_access_devices endpoint."""
    with skip_on_unsupported():
        result = client.sm.get_network_sm_user_access_devices(network_id=network_id).collect()
    assert isinstance(result, list)


def test_get_network_sm_users(client: MerakiClient, network_id: str) -> None:
    """Test get_network_sm_users endpoint."""
    with skip_on_unsupported():
        client.sm.get_network_sm_users(network_id=network_id)


def test_get_organization_sm_admins_roles(client: MerakiClient, organization_id: str) -> None:
    """Test get_organization_sm_admins_roles endpoint."""
    with skip_on_unsupported():
        result = client.sm.get_organization_sm_admins_roles(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_sm_apns_cert(client: MerakiClient, organization_id: str) -> None:
    """Test get_organization_sm_apns_cert endpoint."""
    with skip_on_unsupported():
        client.sm.get_organization_sm_apns_cert(organization_id=organization_id)


def test_get_organization_sm_sentry_policies_assignments_by_network(
    client: MerakiClient, organization_id: str
) -> None:
    """Test get_organization_sm_sentry_policies_assignments_by_network endpoint."""
    with skip_on_unsupported():
        result = client.sm.get_organization_sm_sentry_policies_assignments_by_network(
            organization_id=organization_id
        ).collect()
    assert isinstance(result, list)


def test_get_organization_sm_vpp_accounts(client: MerakiClient, organization_id: str) -> None:
    """Test get_organization_sm_vpp_accounts endpoint."""
    with skip_on_unsupported():
        client.sm.get_organization_sm_vpp_accounts(organization_id=organization_id)
